//-----------------------------------------------------------------------------
// FILE:              JavaEvent.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <sstream>
#include "JavaEvent.h"
#include "JavaListener.h"
#include "util/Trace.h"
#include "util/tstring.h"
#include "com_softtrac_jni_NativeEvent.h"

using namespace std;

namespace SoftTrac {

static Trace trace("JavaEvent", Trace::OFF);


//-----------------------------------------------------------------------------
// JavaEvent::event
//-----------------------------------------------------------------------------
JavaEvent* JavaEvent::event(const std::string& eventName)
{
   static const string METHOD = "event";
   trace.debug(METHOD, toString(eventName));
   JavaMessage* message = JavaMessage::message(eventName);
   JavaEvent* event = dynamic_cast<JavaEvent*>(message);
   trace.debug(METHOD, toString(event));

   return event;
}


//-----------------------------------------------------------------------------
// JavaEvent::JavaEvent
//-----------------------------------------------------------------------------
JavaEvent::JavaEvent(const string& nativeEventName)
   :
      JavaMessage(nativeEventName)
{
}


} // namespace SoftTrac


//-----------------------------------------------------------------------------
// Java_com_softtrac_jni_NativeEvent
//-----------------------------------------------------------------------------
JNIEXPORT jstring JNICALL Java_com_softtrac_jni_NativeEvent(JNIEnv* env, 
                                                            jobject object, 
                                                            jstring eventName, 
                                                            jstring listenerName, 
                                                            bool    attaching)
{
   using namespace SoftTrac;

   static Trace trace(""/*, Trace::ON*/);
   static const string FUNCTION = "Java_com_softtrac_jni_NativeEvent";
   trace.enter(FUNCTION);

   JavaMessage::Sentry sentry(env);

   //---
   // Make copy of event and listener name passed in from java.
   //---
   jboolean owner1, owner2;
   const char* myEventName = env->GetStringUTFChars(eventName, &owner1);
   const char* myListenerName = env->GetStringUTFChars(listenerName, &owner2);
   trace.debug(FUNCTION, myEventName);
   trace.debug(FUNCTION, myListenerName);

   //---
   // Map the event and listener names to their corresponding c++ objects.
   //---
   JavaEvent* event = JavaEvent::event(myEventName);
   JavaListener* listener = JavaListener::listener(myListenerName);

   string returnArgs;
   if (event && listener) 
   {
      //---
      // If we are attaching, store the jni environment pointer and jni object
      // in the listener to connect the c++ listener to the java listener. Then
      // attach the c++ listener to the c++ event.
      //---
      if (attaching)
      {
         listener->setEvent(object);
         event->attach(listener);
      }
      //---
      // If we are detaching, first detach the c++ listener from the c++ event.
      // Then release the jni environment pointer and jni object in the listener 
      // so they are available to be garbage collected. Then detach the c++ 
      // listener from the c++ event.
      //---
      else
      {
         event->detach(listener);
         listener->clearEvent();
      }
      returnArgs = event->formatReturnArgs();
   }
   else returnArgs = "null";

   //---
   // Now that we have a deep copy of the event and listener names, 
   // we can release them on the java side.
   //---
   if (owner1 == JNI_TRUE) 
      env->ReleaseStringUTFChars(eventName, myEventName);

   if (owner2 == JNI_TRUE) 
      env->ReleaseStringUTFChars(listenerName, myListenerName);

   //---
   // Send the event return arguments (if any) back to java.
   //---
   trace.exit(FUNCTION);
   return env->NewStringUTF(returnArgs.c_str());
}


//-----------------------------------------------------------------------------
// Java_com_softtrac_jni_NativeEvent_attach
//-----------------------------------------------------------------------------
JNIEXPORT jstring JNICALL Java_com_softtrac_jni_NativeEvent_attach
  (JNIEnv* env, jobject object, jstring eventName, jstring listenerName)
{
   bool attaching = true;
   
   return Java_com_softtrac_jni_NativeEvent(env, 
                                            env->NewGlobalRef(object), 
                                            eventName, 
                                            listenerName, 
                                            attaching);
}


//-----------------------------------------------------------------------------
// Java_com_softtrac_jni_NativeEvent_detach
//-----------------------------------------------------------------------------
JNIEXPORT jstring JNICALL Java_com_softtrac_jni_NativeEvent_detach
  (JNIEnv* env, jobject object, jstring eventName, jstring listenerName)
{
   bool attaching = false;

   return Java_com_softtrac_jni_NativeEvent(env, 
                                            env->NewGlobalRef(object), 
                                            eventName, 
                                            listenerName, 
                                            attaching);
}
