//-----------------------------------------------------------------------------
// FILE:              JavaMessage.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <math.h>
#include <stdexcept>
#include <sstream>
#include "JavaMessage.h"
#include "util/Trace.h"
#include "util/tstring.h"

using namespace std;

namespace SoftTrac {

JNIEnv* JavaMessage::theJNIEnv = 0;
JavaMessage::JNI_Map* JavaMessage::theMessages = 0;

static Trace trace("JavaMessage", Trace::OFF);


//-----------------------------------------------------------------------------
// JavaMessage::~JavaMessage
//-----------------------------------------------------------------------------
JavaMessage::~JavaMessage()
{
}


//-----------------------------------------------------------------------------
// JavaMessage::message
//-----------------------------------------------------------------------------
JavaMessage* JavaMessage::message(const string& nativeClassName)
{
   ostringstream oss;
   oss << nativeClassName << " { } ";
   istringstream iss(oss.str());
   return message(iss);
}


//-----------------------------------------------------------------------------
// JavaMessage::message
//-----------------------------------------------------------------------------
JavaMessage* JavaMessage::message(istream& iss)
{
   static const char* const METHOD = "message";

   string nativeClassName;
   iss >> nativeClassName;
   trace.debug(METHOD, nativeClassName);   
   if (!theMessages) theMessages = new JNI_Map;

   if (trace.debug())
   {
      std::ostringstream registry;
      registry << "Registered messages:\n";
      for (JNI_Map::iterator i  = theMessages->begin(); 
                             i != theMessages->end();
                           ++i)
      {
         registry << (*i).first << " " << (void*)(*i).second << '\n';
      }
      trace.debug(METHOD, registry.str());
   }

   JNI_Map::iterator i = theMessages->find(nativeClassName);
   return (i == theMessages->end()) ? 0 : (*i).second;
}


//-----------------------------------------------------------------------------
// JavaMessage::JavaMessage
//-----------------------------------------------------------------------------
JavaMessage::JavaMessage(const string& nativeClassName)
   :
      theNativeClassName(nativeClassName)
{
   if (!theMessages) theMessages = new JNI_Map;
   (*theMessages)[theNativeClassName] = this;
   trace.debug("JavaMessage", theNativeClassName+" registered");
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, bool value)
{
   put(keyword, (long)value);
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, char value)
{
   put(keyword, (long)value);
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, int value)
{
   put(keyword, (long)value);
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, long value)
{
   put(keyword, toString(value));
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, float value)
{
   put(keyword, (double)value);
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, double value)
{
   put(keyword, toString(value));
}


//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, const char* value)
{
   put(keyword, string(value));
}

   
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string& keyword, 
                      const std::string& value)
{
   theReturnArgs[keyword] = value;
}

   
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string&       keyword, 
                      const std::vector<bool>& value)
{
   theReturnArgs[keyword] = writeContainer(value, long());
}

   
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string&      keyword, 
                      const std::vector<int>& value)
{
   theReturnArgs[keyword] = writeContainer(value, long());
}

   
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string&       keyword, 
                      const std::vector<long>& value)
{
   theReturnArgs[keyword] = writeContainer(value, long());
}

   
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string&        keyword, 
                      const std::vector<float>& value)
{
   theReturnArgs[keyword] = writeContainer(value, double());
}

   
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(const std::string&         keyword, 
                      const std::vector<double>& value)
{
   theReturnArgs[keyword] = writeContainer(value, double());
}

      
//-----------------------------------------------------------------------------
// JavaMessage::put
//-----------------------------------------------------------------------------
void JavaMessage::put(void* reference)
{
   put("this", reinterpret_cast<long>(reference));
}

   
//-----------------------------------------------------------------------------
// JavaMessage::getBool
//-----------------------------------------------------------------------------
bool JavaMessage::getBool(const std::string& keyword)
{
   long value = getLong(keyword);
   return (bool)value;
}


//-----------------------------------------------------------------------------
// JavaMessage::getChar
//-----------------------------------------------------------------------------
char JavaMessage::getChar(const std::string& keyword)
{
   long value = getLong(keyword);
   return (char)value;
}


//-----------------------------------------------------------------------------
// JavaMessage::getInt
//-----------------------------------------------------------------------------
int JavaMessage::getInt(const std::string& keyword)
{
   long value = getLong(keyword);
   return (int)value;
}


//-----------------------------------------------------------------------------
// JavaMessage::getLong
//-----------------------------------------------------------------------------
long JavaMessage::getLong(const std::string& keyword)
{
   string value = theMethodArgs[keyword];
   return atol(value.c_str());
}


//-----------------------------------------------------------------------------
// JavaMessage::getFloat
//-----------------------------------------------------------------------------
float JavaMessage::getFloat(const std::string& keyword)
{
   double value = getDouble(keyword);
   return (float)value;
}


//-----------------------------------------------------------------------------
// JavaMessage::getDouble
//-----------------------------------------------------------------------------
double JavaMessage::getDouble(const std::string& keyword)
{
   string value = theMethodArgs[keyword];
   return atof(value.c_str());
}


//-----------------------------------------------------------------------------
// JavaMessage::getString
//-----------------------------------------------------------------------------
std::string JavaMessage::getString(const std::string& keyword)
{
   return theMethodArgs[keyword];
}


//-----------------------------------------------------------------------------
// JavaMessage::getBoolArray
//-----------------------------------------------------------------------------
std::vector<bool> JavaMessage::getBoolArray(const std::string& keyword)
{
   vector<bool> v;
   readContainer(v, bool(), theMethodArgs[keyword]);
   return v;
}


//-----------------------------------------------------------------------------
// JavaMessage::getIntArray
//-----------------------------------------------------------------------------
std::vector<int> JavaMessage::getIntArray(const std::string& keyword)
{
   vector<int> v;
   readContainer(v, int(), theMethodArgs[keyword]);
   return v;
}


//-----------------------------------------------------------------------------
// JavaMessage::getLongArray
//-----------------------------------------------------------------------------
std::vector<long> JavaMessage::getLongArray(const std::string& keyword)
{
   vector<long> v;
   readContainer(v, long(), theMethodArgs[keyword]);
   return v;
}


//-----------------------------------------------------------------------------
// JavaMessage::getFloatArray
//-----------------------------------------------------------------------------
std::vector<float> JavaMessage::getFloatArray(const std::string& keyword)
{
   vector<float> v;
   readContainer(v, float(), theMethodArgs[keyword]);
   return v;
}


//-----------------------------------------------------------------------------
// JavaMessage::getDoubleArray
//-----------------------------------------------------------------------------
std::vector<double> JavaMessage::getDoubleArray(const std::string& keyword)
{
   vector<double> v;
   readContainer(v, double(), theMethodArgs[keyword]);
   return v;
}


//-----------------------------------------------------------------------------
// JavaMessage::getReference
//-----------------------------------------------------------------------------
void* JavaMessage::getReference()
{
   unsigned long ptr = getLong("this");
   return reinterpret_cast<void*>(ptr);
}


//-----------------------------------------------------------------------------
// JavaMessage::parseMethodArgs
//-----------------------------------------------------------------------------
void JavaMessage::parseMethodArgs(istream& iss)
{
   static const string METHOD = "parseMethodArgs";
   string start, key;
   iss >> start >> key;
   if (start != "{") throw invalid_argument("Invalid start character "+start);

   while (key != "}")
   {
      // Throw out tab
      char tab = iss.get();
      if (tab != '\t') throw invalid_argument("Invalid tab delimiter "+tab);
   
      //---
      // Read the rest of the line
      //---
      char buffer[1024];
      iss.getline(buffer, 1024);
      string value = buffer;
      theMethodArgs[key] = value;
      trace.debug(METHOD, key+":\t"+value);
      iss >> key;
   }

   if (key != "}") throw invalid_argument("Invalid stop character "+key);
}


//-----------------------------------------------------------------------------
// JavaMessage::formatReturnArgs
//-----------------------------------------------------------------------------
string JavaMessage::formatReturnArgs() const
{
   ostringstream oss;
   oss << theNativeClassName << " {\n";
   for (KV_Map::const_iterator i  = theReturnArgs.begin(); 
                               i != theReturnArgs.end(); 
                             ++i)
   {
      oss << (*i).first << "\t" << (*i).second << "\n";
   }
   oss << "}\t";
   trace.debug("formatReturnArgs", oss.str());

   return oss.str();
}


} // namespace SoftTrac
