//-----------------------------------------------------------------------------
// FILE:              NativeCommand.java
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------

package com.softtrac.jni;

import java.util.*;
import com.softtrac.util.Memento;
import com.softtrac.util.Trace;

/**
 * The NativeCommand class is an abstraction for a native method call.
 * Derived classes act as function objects to execute a method on a
 * native platform. Parameters are passed to and from the native platform
 * as simple keyword value strings.
 *
 * @author Timothy R. Culp
 * @version %W% %G%
 */
public abstract class NativeCommand extends NativeMessage {
   //>
   /**
    * The execute method will execute the native method. The default
    * behavior of execute is to call the preExecute method to give
    * the derived class an opportunity to store arguments to be passed
    * to the native method. Arguments are stored in the base class
    * by making successive calls to the various put methods. Arguments
    * will be formatted into a message with this format:
    * <PRE>
    *    className {
    *    keyword1   value1
    *    keyword2   value2
    *    ...
    *    }
    * </PRE>
    * This message is passed to the native platform by calling the
    * NativeCommand call. This is a native function that takes a String
    * and returns a String.
    *
    * After NativeCommand returns, the String is parsed into keyword
    * value pairs. The postExecute method is called to give the
    * derived classes an opportunity the retrieve arguments by making
    * successive calls to the various get methods.
    *
    * @throws NativeCommand.Exception In the event a command is not understood
    * or the return arguments are not in a recognizable format.
    */
   //<

   public void execute() throws NativeCommand.Exception {
      //>
      trace.enter("execute");
      
      // Give the subclass an opportunity to add arguments
      preExecute();

      // Format the arguments
      String methodArgs = formatMethodArgs();
      
      //---
      // Invoke the native method with the arguments
      //---
      trace.debug("execute", methodArgs);
      String returnArgs = execute(methodArgs);
      trace.debug("execute", returnArgs);

      // Parse the return and store the keyword-value pairs
      parseReturnArgs(returnArgs);

      // Give the derived classes and chance to process the return values
      postExecute();
      
      trace.exit("execute");
      //<
   }
   //>
   /**
    * The addNativeListener method attaches a listener to a native subject.
    * @throws NativeCommand.Exception In the event there is no subject for
    * which to attach.
    */
   //<
   protected NativeCommand(String remoteProxy) {
      //>
      super(remoteProxy);
      //<
   }
   protected void preExecute(){}

   protected void postExecute(){}

   protected abstract String nativeProxy();

   private native String execute(String commandArgs);   
   
   private static Trace trace;
   static {
      //>
      trace = new Trace("jni.NativeCommand");
      // trace.setAll();
      //<
   }
}


   








