//-----------------------------------------------------------------------------
// FILE:              NativeMessage.java
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------

package com.softtrac.jni;

import java.util.*;
import com.softtrac.util.StringParser;
import com.softtrac.util.Memento;
import com.softtrac.util.Trace;

/**
 * The NativeMessage class is an abstraction for a native method call.
 * Derived classes act as function objects to execute a method on a
 * native platform. Parameters are passed to and from the native platform
 * as simple keyword value strings.
 *
 * @author Timothy R. Culp
 * @version %W% %G%
 */
public abstract class NativeMessage {
   //>
   /**
    * The execute method will execute the native method. The default
    * behavior of execute is to call the preExecute method to give
    * the derived class an opportunity to store arguments to be passed
    * to the native method. Arguments are stored in the base class
    * by making successive calls to the various put methods. Arguments
    * will be formatted into a message with this format:
    * <PRE>
    *    className {
    *    keyword1   value1
    *    keyword2   value2
    *    ...
    *    }
    * </PRE>
    * This message is passed to the native platform by calling the
    * NativeMessage call. This is a native function that takes a String
    * and returns a String.
    *
    * After NativeMessage returns, the String is parsed into keyword
    * value pairs. The postExecute method is called to give the
    * derived classes an opportunity the retrieve arguments by making
    * successive calls to the various get methods.
    *
    * @throws NativeMessage.Exception In the event a command is not
    * understood or the return arguments are not in a recognizable format.
    */
   //<

   public void put(String keyword, boolean value) {
      //>
      long lvalue = (value) ? 1 : 0;
      put(keyword, lvalue);
   } //<
   public void put(String keyword, char value) {
      //>
      put(keyword, (long)value);
   } //<
   public void put(String keyword, int value) {
      //>
      put(keyword, (long)value);
   } //<
   public void put(String keyword, long value) {
      //>
      put(keyword, (new Long(value)).toString());
   } //<
   public void put(String keyword, float value) {
      //>
      put(keyword, (double)value);
   } //<
   public void put(String keyword, double value) {
      //>
      put(keyword, (new Double(value)).toString());
   } //<

   public void put(String keyword, String value) {
      //>
      theMethodArgs.put(keyword, value);
   } //<

   public void put(String keyword, boolean[] value) {
      //>
      put(keyword, booleanToLongArray(value));
   } //<
   public void put(String keyword, char[] value) {
      //>
      put(keyword, new String(value));
   } //<
   public void put(String keyword, int[] value) {
      //>
      put(keyword, intToLongArray(value));
   } //<
   public void put(String keyword, long[] value) {
      //>
      String svalue = ""+value.length+" [ ";
      for (int i = 0; i < value.length; ++i) svalue += ""+value[i]+" ";
      svalue += "]";
      
      put(keyword, svalue);
   } //<
   public void put(String keyword, float[] value) {
      //>
      put(keyword, floatToDoubleArray(value));
   } //<
   public void put(String keyword, double[] value) {
      //>
      String svalue = ""+value.length+" [ ";
      for (int i = 0; i < value.length; ++i) svalue += ""+value[i]+" ";
      svalue += "]";
      
      put(keyword, svalue);
   } //<

   public void put(String keyword, Memento value) {
      //>
      theMethodArgs.put(keyword, value.saveTo());
   } //<
   public void put(long reference) {
      //>
      put("this", reference);
   } //<

   public boolean   getBoolean(String keyword) {
      //>
      return (getLong(keyword) == 0) ? false : true;
   } //<
   public char      getChar(String keyword) {
      //>
      return (char)getLong(keyword);
   } //<
   public int       getInt(String keyword) {
      //>
      return (int)getLong(keyword);
   } //<
   public long      getLong(String keyword) {
      //>
      String value = (String)theReturnArgs.get(keyword);
      Long object = new Long(value);
      return object.longValue();
   } //<
   public float     getFloat(String keyword) {
      //>
      return (float)getDouble(keyword);
   } //<
   public double    getDouble(String keyword) {
      //>
      String value = (String)theReturnArgs.get(keyword);
      Double object = new Double(value);
      return object.doubleValue();
   } //<

   public String    getString(String keyword) {
      //>
      return (String)theReturnArgs.get(keyword);
   } //<

   public boolean[] getBooleanArray(String keyword) throws BadReturn {
      //>
      long[] array = getLongArray(keyword);
      return longToBooleanArray(array);
   } //<
   public int[]     getIntArray(String keyword) throws BadReturn {
      //>
      long[] array = getLongArray(keyword);
      return longToIntArray(array);
   } //<
   public long[]    getLongArray(String keyword) throws BadReturn {
      //>
      String value = (String)theReturnArgs.get(keyword);

      //---
      // Tokenize the return value
      //---
      StringTokenizer tokens = new StringTokenizer(value);
      int numTokens = tokens.countTokens();
      Integer length = new Integer(tokens.nextToken());

      //---
      // Check the length
      //---
      if (length.intValue() <= 0)
         throw new BadReturn("Invalid length "+length.intValue());

      //---
      // Check the number of tokens
      //---
      if (numTokens != length.intValue() + 3)
      {
         throw new BadReturn("Invalid number of tokens "
                             + numTokens
                             + " for array of length "
                             + length.intValue());
      }
      
      // Allocate output array
      long[] oarray = new long[length.intValue()];

      //---
      // Check the start character
      //---
      String start = tokens.nextToken();
      if (!start.equals("["))
         throw new BadReturn("Invalid start character "+start);

      //---
      // Fill the array
      //---
      for (int i = 0; i < length.intValue(); ++i) 
      {
         Long v = new Long(tokens.nextToken());
         oarray[i] = v.longValue();
      }
      
      //---
      // Check the start character
      //---
      String stop = tokens.nextToken();
      if (!stop.equals("]"))
         throw new BadReturn("Invalid stop character "+stop);

      return oarray;
   } //<
   public float[]   getFloatArray(String keyword) throws BadReturn {
      //>
      double[] array = getDoubleArray(keyword);
      return doubleToFloatArray(array);
   } //<
   public double[]  getDoubleArray(String keyword) throws BadReturn {
      //>
      String value = (String)theReturnArgs.get(keyword);

      //---
      // Tokenize the return value
      //---
      StringTokenizer tokens = new StringTokenizer(value);
      int numTokens = tokens.countTokens();
      Integer length = new Integer(tokens.nextToken());

      //---
      // Check the length
      //---
      if (length.intValue() <= 0)
         throw new BadReturn("Invalid length "+length.intValue());

      //---
      // Check the number of tokens
      //---
      if (numTokens != length.intValue() + 3)
      {
         throw new BadReturn("Invalid number of tokens "
                             + numTokens
                             + " for array of length "
                             + length.intValue());
      }
      
      // Allocate output array
      double[] oarray = new double[length.intValue()];

      //---
      // Check the start character
      //---
      String start = tokens.nextToken();
      if (!start.equals("["))
         throw new BadReturn("Invalid start character "+start);

      //---
      // Fill the array
      //---
      for (int i = 0; i < length.intValue(); ++i) 
      {
         Double v = new Double(tokens.nextToken());
         oarray[i] = v.doubleValue();
      }
      
      //---
      // Check the start character
      //---
      String stop = tokens.nextToken();
      if (!stop.equals("]"))
         throw new BadReturn("Invalid stop character "+stop);

      return oarray;
   } //<

   public String    getMemento(String keyword) {
      //>
      return getString(keyword);
   } //<
   public long      getReference() {
      //>
      return getLong("this");
   } //<

   public class Exception extends java.lang.Exception {
      public Exception(String type, String message) {
         //>
         super(type+" - "+message);
      } //<
   } 
   public class BadMethod extends Exception {
      public BadMethod(String className) {
         //>
         super("BadMethod", className);
      } //<
   } 
   public class BadReturn extends Exception {
      public BadReturn() {
         //>
         super("BadReturn", "");
      } //<
      public BadReturn(String message) {
         //>
         super("BadReturn", message);
      } //<
   } 

   protected NativeMessage(String remoteProxy) {
      //>
      theMethodArgs = new TreeMap();
      theReturnArgs = new TreeMap();
      
      put("remote_proxy", remoteProxy);
   } //<

   protected abstract String nativeProxy();
   
   protected String formatMethodArgs() {
      //>
      String methodArgs = nativeProxy() + " {\n";
      Iterator i = theMethodArgs.entrySet().iterator();
      while (i.hasNext()) {
         Map.Entry entry = (Map.Entry)i.next();
         methodArgs += entry.getKey() + "\t" + entry.getValue() + "\n";
      }
      methodArgs += "}\t";

      trace.debug("formatMethodArgs", methodArgs);
      return methodArgs;
      //<
   }
   protected void parseReturnArgs(String returnArgs)
      throws NativeMessage.Exception {
      //>
      final String METHOD = "parseReturnArgs";
      
      trace.enter(METHOD);
      trace.debug(METHOD, returnArgs);
      
      // Tokenize the return into keyword pairs
      StringParser parser = new StringParser(returnArgs);

      try 
      {
         String className = parser.read('\n');
         if (className.charAt(className.length()-1)!='{')
            throw new BadReturn();

         boolean done = false;
         while (!done)
         {
            String keyword = parser.read('\t');
            if (keyword.equals("}")) {
               done = true;
               break;
            }
               
            String value = parser.read('\n');
            theReturnArgs.put(keyword, value);
         }
      }
      catch (java.io.IOException e)
      {
         throw new BadReturn(e.getMessage());
      }

      trace.exit(METHOD);
   } //<

   private long[]    booleanToLongArray(boolean[] iarray) {
      //>
      long[] oarray = new long[iarray.length];
      for (int i = 0; i < iarray.length; ++i) oarray[i] = (iarray[i]) ? 1 : 0;
      return oarray;
   } //<
   private boolean[] longToBooleanArray(long[]    iarray) {
      //>
      boolean[] oarray = new boolean[iarray.length];
      for (int i = 0; i < iarray.length; ++i) oarray[i] = (iarray[i] == 1);
      return oarray;
   } //<
   private long[]    intToLongArray    (int[]     iarray) {
      //>
      long[] oarray = new long[iarray.length];
      for (int i = 0; i < iarray.length; ++i) oarray[i] = iarray[i];
      return oarray;
   } //<
   private int[]     longToIntArray    (long[]    iarray) {
      //>
      int[] oarray = new int[iarray.length];
      for (int i = 0; i < iarray.length; ++i) oarray[i] = (int)iarray[i];
      return oarray;
   } //<
   private double[]  floatToDoubleArray(float[]   iarray) {
      //>
      double[] oarray = new double[iarray.length];
      for (int i = 0; i < iarray.length; ++i) oarray[i] = iarray[i];
      return oarray;
   } //<
   private float[]   doubleToFloatArray(double[]  iarray) {
      //>
      float[] oarray = new float[iarray.length];
      for (int i = 0; i < iarray.length; ++i) oarray[i] = (float)iarray[i];
      return oarray;
   } //<

   private TreeMap theMethodArgs;
   private TreeMap theReturnArgs;
   
   private static Trace trace;
   static {
      //>
      trace = new Trace("jni.NativeMessage");
      trace.setAll();
      //<
   }
}


   








