//-----------------------------------------------------------------------------
// FILE:              BatteryImpl.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <sstream>
#include "BatteryImpl.h"

namespace SoftTrac {
namespace BatteryImpl {

static Battery* battery = new Battery();

//---
// Register remote commands
//---
static ConstructBattery cmd0("BatteryImpl::ConstructBattery");
static DrawPowerCommand cmd1("BatteryImpl::DrawPowerCommand");


//---
// Register remote events
//---
static FullPowerEvent evt0("BatteryImpl::FullPowerEvent");
static PowerChangeEvent evt1("BatteryImpl::PowerChangeEvent");
static LowPowerEvent evt2("BatteryImpl::LowPowerEvent");
static NoPowerEvent evt3("BatteryImpl::NoPowerEvent");


//---
// Register remote listeners
//---
static FullPowerListener lst0("BatteryImpl::FullPowerListener");
static PowerChangeListener lst1("BatteryImpl::PowerChangeListener");
static LowPowerListener lst2("BatteryImpl::LowPowerListener");
static NoPowerListener lst3("BatteryImpl::NoPowerListener");


//-----------------------------------------------------------------------------
// ConstructBattery::ConstructBattery
//
// This constructor maps this c++ NewCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
ConstructBattery::ConstructBattery(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// ConstructBattery::realExecute
//
// This method executes the NewCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void ConstructBattery::realExecute()
{
   //---
   // Get incoming arguments
   //---
   int startPower = getInt("start_power");

   //---
   // Execute some method
   //---
   battery->setPower(startPower);

   //---
   // Put back any return arguments
   //---
   put(battery);
}


//-----------------------------------------------------------------------------
// DrawPowerCommand::DrawPowerCommand
//
// This constructor maps this c++ DrawPowerCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
DrawPowerCommand::DrawPowerCommand(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// DrawPowerCommand::realExecute
//
// This method executes the DrawPowerCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void DrawPowerCommand::realExecute()
{
   //---
   // Get incoming arguments
   //---
   int amount = getInt("amount");

   //---
   // Execute some method
   //---
   battery->drawPower(amount);

   //---
   // Put back any return arguments
   //---
}


//-----------------------------------------------------------------------------
// FullPowerEvent::FullPowerEvent
//
// This constructor maps this c++ FullPowerEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
FullPowerEvent::FullPowerEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// FullPowerEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void FullPowerEvent::attach(JavaListener* lst)
{
   //---
   // attach the listener
   //---
   battery->attachFullPowerEvent(lst);
}


//-----------------------------------------------------------------------------
// FullPowerEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void FullPowerEvent::detach(JavaListener* lst)
{
   //---
   // detach the listener
   //---
   battery->detachFullPowerEvent(lst);
}


//-----------------------------------------------------------------------------
// PowerChangeEvent::PowerChangeEvent
//
// This constructor maps this c++ PowerChangeEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
PowerChangeEvent::PowerChangeEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// PowerChangeEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void PowerChangeEvent::attach(JavaListener* lst)
{
   //---
   // attach the listener
   //---
   battery->attachPowerChangeEvent(lst);
}


//-----------------------------------------------------------------------------
// PowerChangeEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void PowerChangeEvent::detach(JavaListener* lst)
{
   //---
   // detach the listener
   //---
   battery->detachPowerChangeEvent(lst);
}


//-----------------------------------------------------------------------------
// LowPowerEvent::LowPowerEvent
//
// This constructor maps this c++ LowPowerEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
LowPowerEvent::LowPowerEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// LowPowerEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void LowPowerEvent::attach(JavaListener* lst)
{
   //---
   // attach the listener
   //---
   battery->attachLowPowerEvent(lst);
}


//-----------------------------------------------------------------------------
// LowPowerEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void LowPowerEvent::detach(JavaListener* lst)
{
   //---
   // detach the listener
   //---
   battery->detachLowPowerEvent(lst);
}


//-----------------------------------------------------------------------------
// NoPowerEvent::NoPowerEvent
//
// This constructor maps this c++ NoPowerEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
NoPowerEvent::NoPowerEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// NoPowerEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void NoPowerEvent::attach(JavaListener* lst)
{
   //---
   // attach the listener
   //---
   battery->attachNoPowerEvent(lst);
}


//-----------------------------------------------------------------------------
// NoPowerEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void NoPowerEvent::detach(JavaListener* lst)
{
   //---
   // detach the listener
   //---
   battery->detachNoPowerEvent(lst);
}


//-----------------------------------------------------------------------------
// FullPowerListener::FullPowerListener
//
// This constructor maps this c++ FullPowerListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
FullPowerListener::FullPowerListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// FullPower::update
//
// This constructor maps this c++ FullPowerListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void FullPowerListener::update(Event* event)
   throw (std::invalid_argument)
{
   //---
   // Cast incoming event to the proper subject
   //---
   Battery* myBattery = Battery::Event::cast(event);
   if (!myBattery)
   {
      std::ostringstream what;
      what << event->name() << " events not associated with this subject";
      throw std::invalid_argument(what.str());
   }

   //---
   // Put back any event arguments
   //---
   // put("keyword1", myBattery->value1());
   // put("keyword2", myBattery->value2());

   //---
   // Forward update across the JNI interface
   //---
   forwardNotify();
}


//-----------------------------------------------------------------------------
// PowerChangeListener::PowerChangeListener
//
// This constructor maps this c++ PowerChangeListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
PowerChangeListener::PowerChangeListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// PowerChange::update
//
// This constructor maps this c++ PowerChangeListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void PowerChangeListener::update(Event* event)
   throw (std::invalid_argument)
{
   //---
   // Cast incoming event to the proper subject
   //---
   Battery* myBattery = Battery::Event::cast(event);
   if (!myBattery)
   {
      std::ostringstream what;
      what << event->name() << " events not associated with this subject";
      throw std::invalid_argument(what.str());
   }

   //---
   // Put back any event arguments
   //---
   put("power", myBattery->getPower());

   //---
   // Forward update across the JNI interface
   //---
   forwardNotify();
}


//-----------------------------------------------------------------------------
// LowPowerListener::LowPowerListener
//
// This constructor maps this c++ LowPowerListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
LowPowerListener::LowPowerListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// LowPower::update
//
// This constructor maps this c++ LowPowerListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void LowPowerListener::update(Event* event)
   throw (std::invalid_argument)
{
   //---
   // Cast incoming event to the proper subject
   //---
   Battery* myBattery = Battery::Event::cast(event);
   if (!myBattery)
   {
      std::ostringstream what;
      what << event->name() << " events not associated with this subject";
      throw std::invalid_argument(what.str());
   }

   //---
   // Put back any event arguments
   //---
   put("power", myBattery->getPower());

   //---
   // Forward update across the JNI interface
   //---
   forwardNotify();
}


//-----------------------------------------------------------------------------
// NoPowerListener::NoPowerListener
//
// This constructor maps this c++ NoPowerListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
NoPowerListener::NoPowerListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// NoPower::update
//
// This constructor maps this c++ NoPowerListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void NoPowerListener::update(Event* event)
   throw (std::invalid_argument)
{
   //---
   // Cast incoming event to the proper subject
   //---
   Battery* myBattery = Battery::Event::cast(event);
   if (!myBattery)
   {
      std::ostringstream what;
      what << event->name() << " events not associated with this subject";
      throw std::invalid_argument(what.str());
   }

   //---
   // Put back any event arguments
   //---
   // put("keyword1", myBattery->value1());
   // put("keyword2", myBattery->value2());

   //---
   // Forward update across the JNI interface
   //---
   forwardNotify();
}


} // namespace BatteryImpl
} // namespace SoftTrac
