//----------------------------------------------------------------- 
// FILE: Board.cpp
//----------------------------------------------------------------- 

#include "tconfig.h"
#include <algorithm>
#include <iostream>
#include <utility>
#include <vector>
#include "Board.h"
#include "BoardIterator.h"

using namespace std;

namespace SoftTrac {


int Board::SIZE   = -1;
int Board::SERIES = -1;


//----------------------------------------------------------------- 
// FUNCTION OBJECT: ShortestDistance
//----------------------------------------------------------------- 
class Board::ShortestDistance
   : public std::binary_function<Square,Square,bool>
{
public:
   ShortestDistance(const Square& finish) : theFinish(finish) {}

   bool operator()(const Square& s1, const Square& s2) const
   {
      return s1.distanceTo(theFinish) <= s2.distanceTo(theFinish);
   }  

private:
	Square theFinish;
};


//----------------------------------------------------------------- 
// FUNCTION OBJECT: BadChoice
//----------------------------------------------------------------- 
class Board::BadChoice : public std::unary_function<Square,bool>
{
public:
   BadChoice(const Board&  board, 
             const Square& start) 
      : 
         theBoard(board),
         theStart(start) 
   {}

   bool operator()(const Square& s) const
   {
      return (theBoard.isOutOfBounds(s) ||
              theBoard.isOccupied(s)    ||
              s == theStart);
   }  

private:
   const Board& theBoard;
   Square       theStart;
};


//----------------------------------------------------------------- 
// FUNCTION OBJECT: MarkSeries
//----------------------------------------------------------------- 
class Board::MarkSeries : public std::unary_function<Token*,void>
{
public:
   MarkSeries(Board* b) : theBoard(b), theLastToken(0) {}

   void operator()(Token* currentToken) 
   { theBoard->scan(currentToken, theLastToken); }

private:
   Board* theBoard;
   Token  theLastToken;
};


//----------------------------------------------------------------- 
// Board::Board
//----------------------------------------------------------------- 
Board::Board(int size)
   :
      theEvents   (this),
      theScore    (0),
      theSkillBar (SKILL_BAR)
{
   SIZE   = size;
   SERIES = Board::SIZE/2 + 1;

   // Allocate rows
   theBoard = new Token**[SIZE];

   // For each row, allocate columns
   for (int r = 0; r < SIZE; ++r)
   {
      theBoard[r] = new Token*[SIZE];
    
      // For each col, allocate tokens
      for (int c = 0; c < SIZE; ++c)
      {
         theBoard[r][c] = new Token();
      }
   }
}


//----------------------------------------------------------------- 
// Board::~Board
//----------------------------------------------------------------- 
Board::~Board()
{
   // Deallocate tokens
   for (int r = 0; r < SIZE; ++r)
   {
      for (int c = 0; c < SIZE; ++c)
      {
         delete theBoard[r][c];
      }

      // Deallocate columns
      delete[] theBoard[r];
   }

   // Deallocate rows
   delete[] theBoard;
}


//----------------------------------------------------------------- 
// Board::move
//----------------------------------------------------------------- 
bool Board::move(const Square& start, const Square& finish)
{
   Board::SquareStack p = path(start, finish);
   if (p.empty()) return false;

   Square s = start;
   while (!p.empty())
   {
      Square f = p.top();
      char c = getToken(s)->getToken();
      putToken(s, ' ');
      putToken(f, c);
	   p.pop();
	   s = f;
   }

   theEvents.notify(Board::MOVE_COMPLETE);
   return true;
}


//----------------------------------------------------------------- 
// Board::isOutOfBounds
//----------------------------------------------------------------- 
bool Board::isOutOfBounds(const Square& square) const
{
   return (square.row() < 0     || 
           square.row() >= SIZE ||
           square.col() < 0     ||
           square.col() >= SIZE);
}


//----------------------------------------------------------------- 
// Board::path
//----------------------------------------------------------------- 
Board::SquareStack Board::path(const Square& start, 
                               const Square& finish) const
{
   if (!isOccupied(start) || isOccupied(finish)) return SquareStack();

   //---
   // Find the path from start to finish using the
   // ShortestDistance hueristic
   //---
   ShortestDistance hueristic(finish);
   SquareStack path;
   findPath(start, finish, hueristic, path);

   for (int r = 0; r < SIZE; ++r)
   {
      for (int c = 0; c < SIZE; ++c)
      {
         Square s(r,c);
         if (isMarked(s)) const_cast<Board*>(this)->clear(s);
      }
   }

   //---
   // The path returned is the reverse of what the user
   // wants so let's return the reverse path
   //---
   SquareStack reversePath;
   while (!path.empty())
   {
      reversePath.push(path.top());
      path.pop();
   }

   return reversePath;
}


//----------------------------------------------------------------- 
// Board::Event::Event
//----------------------------------------------------------------- 
Board::Event::Event(Board* board)
   :
      SubjectEvent<Board>(board, "Board", Board::NUM_EVENTS)
{
}


//----------------------------------------------------------------- 
// Board::Event::notifyChange
//----------------------------------------------------------------- 
void Board::Event::notifyChange(const Square& s)
{
   theSquare = s;
   notify(Board::CHANGE);
}


//----------------------------------------------------------------- 
// Board::findPath
//----------------------------------------------------------------- 
bool Board::findPath(const Square&    start, 
                     const Square&    finish,
                     const Hueristic& hueristic,
                     SquareStack&     path) const
{
   // Check to see if we are done
   if (start == finish) return true;

   //---
   // Let's pick four choices
   //---
   typedef std::vector<Square> Choices;
   Choices choices;
   choices.push_back(Square(start.row(),   start.col()-1));
   choices.push_back(Square(start.row(),   start.col()+1));
   choices.push_back(Square(start.row()-1, start.col()));
   choices.push_back(Square(start.row()+1, start.col()));

//	using namespace std;
//	Choices::const_iterator s;
//	cout << "Choices:" << endl;
//	for (s = choices.begin(); s != choices.end(); ++s)
//	{
//	   cout << *s << endl;
//	}

   //---
   // Eliminate the bad ones. Since remove_if does not change 
   // size of our container we need to delete the dangling 
   // references after.
   //---
   BadChoice badChoice(*this, start);
   Choices::iterator result = std::remove_if(choices.begin(), 
                                             choices.end(),
                                             badChoice);
   choices.erase(result, choices.end());

//	Choices::const_iterator s;
//	cout << "Good Choices:" << endl;
//	for (s = choices.begin(); s != choices.end(); ++s)
//	{
//	   cout << *s << endl;
//	}

   // Sort the remaining choices based on our hueristic
   std::sort(choices.begin(), choices.end(), hueristic); 

//	cout << "Sorted Good Choices:" << endl;
//	for (s = choices.begin(); s != choices.end(); ++s)
//	{
//	   cout << *s << '\t' << (*s).distanceTo(finish) << endl;
//	}

   //---
   // Recursively search the choices until the finish
   // square is found or all choices are exhausted
   //---
   bool found = false;
   for (Choices::const_iterator s  = choices.begin();
                                s != choices.end() && !found;
                              ++s)
   {
      path.push(*s);
      const_cast<Board*>(this)->mark(*s);
      found = findPath(*s, finish, hueristic, path);
      if (!found) path.pop();
   }

   return found;
}


//----------------------------------------------------------------- 
// Board::isFull
//----------------------------------------------------------------- 
bool Board::isFull() const
{
   for (int r = 0; r < SIZE; ++r)
   {    
      for (int c = 0; c < SIZE; ++c)
      {
         Token* token = theBoard[r][c];
         if (token->isBlank()) return false;
      }
   }

   return true;
}

//----------------------------------------------------------------- 
// Board::cleanup
//
// Scans the board for a row, column or diagonal of 5 or more
// tokens and removes them. Returns true if any tokens are removed 
// from the board.
//----------------------------------------------------------------- 
bool Board::cleanup()
{
   unsigned int n;
   theMarkStack = TokenStack();

   //---
   // Scan the rows and mark series of tokens to be cleaned
   //---
   for (n = 0; n < numRows(); ++n)
   {
      MarkSeries markSeries(this);
      Row r = row(n);
      std::for_each(r.begin(), r.end(), markSeries);
   }

   //---
   // Scan the cols and mark series of tokens to be cleaned
   //---
   for (n = 0; n < numCols(); ++n)
   {
      MarkSeries markSeries(this);
      Col c = col(n);
      std::for_each(c.begin(), c.end(), markSeries);
   }

   //---
   // Scan the diags and mark series of tokens to be cleaned
   //---
   for (n = 0; n < numDiags(); ++n)
   {
      MarkSeries markSeries(this);
      BackDiag d = backDiag(n);
      std::for_each(d.begin(), d.end(), markSeries);
   }

   //---
   // Scan the diags and mark series of tokens to be cleaned
   //---
   for (n = 0; n < numDiags(); ++n)
   {
      MarkSeries markSeries(this);
      FwdDiag d = fwdDiag(n);
      std::for_each(d.begin(), d.end(), markSeries);
   }

   //---
   // Check to see whether anything was marked for removal
   //---
   theScore += theMarkStack.size();
   bool cleaner = (theMarkStack.size() > 0);
   
   //---
   // Cleanup!
   //---
   while (!theMarkStack.empty())
   {
      theMarkStack.top()->clear();
      theMarkStack.pop();
   }

   if (cleaner) theEvents.notify(CLEANUP);

   if (theScore >= theSkillBar)
   {
      theSkillBar += SKILL_BAR;
      theEvents.notify(LEVEL_CHANGE);
   }

   return cleaner;
}


//----------------------------------------------------------------- 
// Board::scan
//----------------------------------------------------------------- 
void Board::scan(Token* currentToken, Token& lastToken)
{
   //---
   // If the currentToken is not equal to the last token
   // or there is a break in tokens, we need to stop scanning...
   //---
   if ((*currentToken != lastToken) || (currentToken->isBlank()))
   {
      //---
      // If the testStack is greater than or equal to our threshold,
      // we can mark all the squares for removal by putting them on 
      // the markStack...
      //---
      SquareStack::size_type size = theTestStack.size();
      if (size >= SERIES)
      {
         while (!theTestStack.empty())
         {
            theMarkStack.push(theTestStack.top());
            theTestStack.pop();
         }
      }

      //---
      // ...otherwise, flush any squares on the testStack.
      //---
      else if (size > 0)
      {
         theTestStack = TokenStack();
      }
   }

   //---
   // As long as the currentToken is not a blank, we can
   // add it to the testStack. We don't care whether this
   // is the first token on the stack or continuing in a
   // series.
   //---
   if (currentToken->isNotBlank())
   {
      theTestStack.push(currentToken);
   }

   //---
   // Reset lastToken and continue scanning
   //---
   lastToken = *currentToken;
}


//----------------------------------------------------------------- 
// operator<<
//----------------------------------------------------------------- 
std::ostream& operator<<(std::ostream& os, const Board& b)
{
   for (int col = 0; col < b.size(); ++col) os << "   " << col;
   os << std::endl;

   for (int i = 0; i < b.size(); ++i)
   {
      os << i << ' ';
      for (int j = 0; j < b.size(); ++j)
      {
         os << ' ' << *b.getToken(Square(i,j)) << ' ';
         if (j < b.size()-1) os << '|';
      }
		
      os << "\n  ";

      if (i < b.size()-1)
      {
         for (int k = 0; k < b.size(); ++k)
         {
            os << "---";
            if (k < b.size()-1) os << '-';
         }

         os << '\n';
      }
   }

   return os;
}


} // namespace SoftTrac
