//----------------------------------------------------------------- 
// FILE: Board.h
//----------------------------------------------------------------- 
#ifndef Board_h
#define Board_h

#include <functional>
#include <iosfwd>
#include <stack>
#include <vector>
#include "BoardIterator.h"
#include "Square.h"
#include "Token.h"
#include "util/Event.h"

namespace SoftTrac {


//----------------------------------------------------------------- 
// CLASS: Board
//----------------------------------------------------------------- 
class Board
{
public:
   Board(int size); 
  ~Board();

   unsigned int size()     const { return SIZE; }
   unsigned int numRows()  const { return SIZE; }
   unsigned int numCols()  const { return SIZE; }
   unsigned int numDiags() const { return 2*SIZE-1; }

   const Token* getToken(const Square&) const;
   Token*       getToken(const Square&);

   void putToken(const Square&, char);

   int getScore() const { return theScore; }

   bool move(const Square& start, const Square& finish);

   bool isOutOfBounds(const Square&) const;
   bool isOccupied(const Square&) const;
   bool isMarked(const Square&) const;
   bool isFull() const; 

   void clear(const Square&);
   void mark (const Square&);

   typedef std::stack<Square, std::vector<Square> > SquareStack;

   SquareStack path(const Square& start, const Square& finish) const;

   bool cleanup();

   Row      row     (unsigned int n) { return Row     (this, n); }
   Col      col     (unsigned int n) { return Col     (this, n); }
   BackDiag backDiag(unsigned int n) { return BackDiag(this, n); }
   FwdDiag  fwdDiag (unsigned int n) { return FwdDiag (this, n); }

   void attachChangeEvent(Observer* o) {theEvents.attach(o,CHANGE);}
   void attachMoveCompleteEvent(Observer* o) {theEvents.attach(o,MOVE_COMPLETE);}
   void attachCleanupEvent(Observer* o) {theEvents.attach(o,CLEANUP);}
   void attachLevelChangeEvent(Observer* o) {theEvents.attach(o,LEVEL_CHANGE);}
 
   void detachChangeEvent(Observer* o) {theEvents.detach(o,CHANGE);}
   void detachMoveCompleteEvent(Observer* o) {theEvents.detach(o,MOVE_COMPLETE);}
   void detachCleanupEvent(Observer* o) {theEvents.detach(o,CLEANUP);}
   void detachLevelChangeEvent(Observer* o) {theEvents.detach(o,LEVEL_CHANGE);}

   enum 
   {
      CHANGE        = 0,
      MOVE_COMPLETE = 1,
      CLEANUP       = 2,
      LEVEL_CHANGE  = 3,
      NUM_EVENTS    = 4
   };

   class Event : public SubjectEvent<Board>
   {
   public:
      Event(Board* b);
      void notifyChange(const Square& s);

      const Square& square() const { return theSquare; }

   private:
      Square theSquare;
   };

private:
   class ShortestDistance;
   class BadChoice;
   class MarkSeries;

   typedef ShortestDistance Hueristic;

   bool findPath(const Square&    start,
                 const Square&    finish,
                 const Hueristic& hueristic,
                 SquareStack&     path) const;

   static int SIZE;
   static int SERIES;

   enum { SKILL_BAR = 500 };

   void scan(Token* currentToken, Token& lastToken);

   typedef std::stack<Token*, std::vector<Token*> > TokenStack;

   mutable Token*** theBoard;
   int              theScore;
   int              theSkillBar;
   TokenStack       theTestStack;
   TokenStack       theMarkStack;
   Event            theEvents;

   friend class MarkSeries;
};


std::ostream& operator<<(std::ostream&, const Board&);


//----------------------------------------------------------------- 
// Board::getToken
//----------------------------------------------------------------- 
inline const Token* Board::getToken(const Square& s) const 
{ 
   return theBoard[s.row()][s.col()]; 
}


//----------------------------------------------------------------- 
// Board::getToken
//----------------------------------------------------------------- 
inline Token* Board::getToken(const Square& s)       
{ 
   return theBoard[s.row()][s.col()]; 
}


//----------------------------------------------------------------- 
// Board::putToken
//----------------------------------------------------------------- 
inline void Board::putToken(const Square& s, char c) 
{ 
   theBoard[s.row()][s.col()]->setToken(c); 
   theEvents.notifyChange(s);
}


//----------------------------------------------------------------- 
// Board::isOccupied
//----------------------------------------------------------------- 
inline bool Board::isOccupied(const Square& s) const
{ 
   return theBoard[s.row()][s.col()]->isNotBlank(); 
}


//----------------------------------------------------------------- 
// Board::isMarked
//----------------------------------------------------------------- 
inline bool Board::isMarked(const Square& s) const
{ 
   return theBoard[s.row()][s.col()]->isMarked(); 
}


//----------------------------------------------------------------- 
// Board::clear
//----------------------------------------------------------------- 
inline void Board::clear(const Square& s)
{ 
   theBoard[s.row()][s.col()]->clear(); 
}


//----------------------------------------------------------------- 
// Board::mark
//----------------------------------------------------------------- 
inline void Board::mark(const Square& s)
{ 
   theBoard[s.row()][s.col()]->mark(); 
}


} // namespace SoftTrac


#endif
