package com.softtrac.nomess;

import com.softtrac.jni.*;
import com.softtrac.util.Trace;
import com.softtrac.util.Assertion;

public class Board
{
   public static final int Change       = 0;
   public static final int MoveComplete = 1;
   public static final int Cleanup      = 2;
   public static final int NumEvents    = 3;

   public Board(int size) throws NativeMessage.Exception {
      //>
      final String METHOD = "Board";
      trace.enter(METHOD);

      theSize = size;
      
      //---
      // Create a native Board
      //---
      NativeCommand command = new ConstructCommand();
      command.execute();

      //---
      // Create the list of events
      //---
      theEvents = new NativeEvent[NumEvents];
      theEvents[Change]       = new ChangeEvent();
      theEvents[MoveComplete] = new MoveCompleteEvent();
      theEvents[Cleanup]      = new CleanupEvent();

      trace.exit(METHOD);
   } //<

   public int getSize() { return theSize; }
   
   public long reference() { return theBoard; }

   public boolean isOccupied(Square square)
      throws NativeMessage.Exception {
      //>
      IsOccupiedCommand command = new IsOccupiedCommand(square);
      command.execute();
      return command.isOccupied();
   } //<
   
   public boolean move(Square start, Square finish)
      throws NativeMessage.Exception {
      //>
      MoveCommand command = new MoveCommand(start, finish);
      command.execute();
      return command.isValid();      
   } //<
   
   public void addNativeListener(NativeListener listener)
      throws NativeMessage.Exception {
      //>
      final String METHOD = "addNativeListener";
      trace.enter(METHOD);

      //---
      // Get event number from listener
      //---
      int eventNo = listener.eventNo();
      new Assertion(eventNo >= 0 && eventNo < NumEvents); 

      //---
      // Get the event and add the listener
      //---
      NativeEvent event = theEvents[eventNo];
      event.addNativeListener(listener);
      
      trace.exit(METHOD);
   } //<

   //---
   // Native commands
   //---
   class ConstructCommand extends NativeCommand {
      ConstructCommand() {
         //>
         super("Board.ConstructCommand");
         
         //---
         // Put any command arguments
         //---
         put("size", theSize);
      } //<
      public void execute() throws NativeMessage.Exception {
         //>
         super.execute();
         
         //---
         // Get any return arguments
         //---
         theBoard = getReference();
      } //<
      protected String nativeProxy() {
         //>
         return "BoardServer::ConstructCommand";
      }
      //<
   }

   class IsOccupiedCommand extends NativeCommand {
      IsOccupiedCommand(Square square) {
         //>
         super("Board.IsOccupiedCommand");
         
         //---
         // Put any command arguments
         //---
         put("row", square.row());
         put("col", square.col());
       } //<
      public void execute() throws NativeMessage.Exception {
         //>
         super.execute();
         
         //---
         // Get any return arguments
         //---
         theSquareIsOccupied = getBoolean("occupied");
      } //<
      protected String nativeProxy() {
         //>
         return "BoardServer::IsOccupiedCommand";
      }
      //<
      boolean isOccupied() { return theSquareIsOccupied; }
      private boolean theSquareIsOccupied;
   }
   
   class MoveCommand extends NativeCommand {
      MoveCommand(Square start, Square finish) {
         //>
         super("Board.MoveCommand");
         
         //---
         // Put any command arguments
         //---
         put("start",  start.data());
         put("finish", finish.data());
       } //<
      public void execute() throws NativeMessage.Exception {
         //>
         super.execute();
         
         //---
         // Get any return arguments
         //---
         theMoveIsValid = getBoolean("valid");
      } //<
      protected String nativeProxy() {
         //>
         return "BoardServer::MoveCommand";
      }
      //<
      boolean isValid() { return theMoveIsValid; }
      private boolean theMoveIsValid;
   }

   //---
   // Native events
   //---
   class ChangeEvent extends NativeEvent {
      ChangeEvent() {
         //>
         super("Board.ChangeEvent");
      } //<
      protected String nativeProxy() {
         //>
         return "BoardServer::ChangeEvent";
      }
      //<
   }

   class MoveCompleteEvent extends NativeEvent {
      MoveCompleteEvent() {
         //>
         super("Board.MoveCompleteEvent");
      } //<
      protected String nativeProxy() {
         //>
         return "BoardServer::MoveCompleteEvent";
      }
      //<
   }

   class CleanupEvent extends NativeEvent {
      CleanupEvent() {
         //>
         super("Board.CleanupEvent");
      } //<
      protected String nativeProxy() {
         //>
         return "BoardServer::CleanupEvent";
      }
      //<
   }

   private NativeEvent[] theEvents;
   private long          theBoard;
   private int           theSize;

   private static Trace trace;
   static {
      //>
      System.loadLibrary("Debug/nomess");
      trace = new Trace("nomess.Board");
      // trace.setAll();
   } //<
}








