//----------------------------------------------------------------- 
// FILE: BoardIterator.h
//----------------------------------------------------------------- 
#ifndef BoardIterator_h
#define BoardIterator_h

#include "Square.h"

namespace SoftTrac {


class Board;
class Token;
class RowIterator;
class ColIterator;
class BackDiagIterator;
class FwdDiagIterator;


//----------------------------------------------------------------- 
// CLASS: Row
//----------------------------------------------------------------- 
class Row
{
public:
   Row(Board* b, unsigned int n);

   typedef RowIterator iterator;

   iterator begin();
   iterator end();

private:
   Board*       theBoard;
   unsigned int theRowNo;
};


//----------------------------------------------------------------- 
// CLASS: Col
//----------------------------------------------------------------- 
class Col
{
public:
   Col(Board* b, unsigned int n);

   typedef ColIterator iterator;

   iterator begin();
   iterator end();

private:
   Board*       theBoard;
   unsigned int theColNo;
};


//----------------------------------------------------------------- 
// CLASS: BackDiag
//----------------------------------------------------------------- 
class BackDiag
{
public:
   BackDiag(Board* b, unsigned int n);

   typedef BackDiagIterator iterator;

   iterator begin();
   iterator end();

private:
   Board* theBoard;
   Square theBegin;
   Square theEnd;
};


//----------------------------------------------------------------- 
// CLASS: FwdDiag
//----------------------------------------------------------------- 
class FwdDiag
{
public:
   FwdDiag(Board* b, unsigned int n);

   typedef FwdDiagIterator iterator;

   iterator begin();
   iterator end();

private:
   Board* theBoard;
   Square theBegin;
   Square theEnd;
};


//----------------------------------------------------------------- 
// CLASS: BoardIterator
//----------------------------------------------------------------- 
class BoardIterator 
{
public:
   bool operator==(const BoardIterator& rp) const;
   bool operator!=(const BoardIterator& rp) const;

   const Token* operator*() const;
   Token*       operator*();
      
protected:
   BoardIterator(Board* b, const Square& s);

   Board* theBoard;
   Square theSquare;
};


//----------------------------------------------------------------- 
// CLASS: RowIterator
//----------------------------------------------------------------- 
class RowIterator : public BoardIterator
{
public:
   RowIterator(Board* b, const Square& s);
      
   RowIterator& operator++();    // prefix
   RowIterator  operator++(int); // postfix
};


//----------------------------------------------------------------- 
// CLASS: ColIterator
//----------------------------------------------------------------- 
class ColIterator : public BoardIterator
{
public:
   ColIterator(Board* b, const Square& s);
      
   ColIterator& operator++();    // prefix
   ColIterator  operator++(int); // postfix
};


//----------------------------------------------------------------- 
// CLASS: BackDiagIterator
//----------------------------------------------------------------- 
class BackDiagIterator : public BoardIterator
{
public:
   BackDiagIterator(Board* b, const Square& s);
      
   BackDiagIterator& operator++();    // prefix
   BackDiagIterator  operator++(int); // postfix
};


//----------------------------------------------------------------- 
// CLASS: FwdDiagIterator
//----------------------------------------------------------------- 
class FwdDiagIterator : public BoardIterator
{
public:
   FwdDiagIterator(Board* b, const Square& s);
      
   FwdDiagIterator& operator++();    // prefix
   FwdDiagIterator  operator++(int); // postfix
};


//----------------------------------------------------------------- 
// Row::begin
//----------------------------------------------------------------- 
inline Row::iterator Row::begin() 
{ 
   return RowIterator(theBoard, Square(theRowNo, 0)); 
}


//----------------------------------------------------------------- 
// Col::begin
//----------------------------------------------------------------- 
inline Col::iterator Col::begin() 
{ 
   return ColIterator(theBoard, Square(0, theColNo)); 
}


//----------------------------------------------------------------- 
// BackDiag::begin
//----------------------------------------------------------------- 
inline BackDiag::iterator BackDiag::begin() 
{ 
   return BackDiagIterator(theBoard, theBegin); 
}


//----------------------------------------------------------------- 
// BackDiag::end
//----------------------------------------------------------------- 
inline BackDiag::iterator BackDiag::end()   
{ 
   return BackDiagIterator(theBoard, theEnd); 
}


//----------------------------------------------------------------- 
// FwdDiag::begin
//----------------------------------------------------------------- 
inline FwdDiag::iterator FwdDiag::begin() 
{ 
   return FwdDiagIterator(theBoard, theBegin); 
}


//----------------------------------------------------------------- 
// FwdDiag::end
//----------------------------------------------------------------- 
inline FwdDiag::iterator FwdDiag::end()   
{ 
   return FwdDiagIterator(theBoard, theEnd); 
}


//----------------------------------------------------------------- 
// BoardIterator::operator==
//----------------------------------------------------------------- 
inline 
bool BoardIterator::operator==(const BoardIterator& rp) const 
{ 
   return theBoard==rp.theBoard && theSquare==rp.theSquare; 
}


//----------------------------------------------------------------- 
// BoardIterator::operator!=
//----------------------------------------------------------------- 
inline 
bool BoardIterator::operator!=(const BoardIterator& rp) const 
{ 
   return !(*this == rp); 
}


//----------------------------------------------------------------- 
// BoardIterator::BoardIterator
//----------------------------------------------------------------- 
inline BoardIterator::BoardIterator(Board* b, const Square& s) 
   : 
      theBoard(b), theSquare(s) 
{
}


//----------------------------------------------------------------- 
// RowIterator::RowIterator
//----------------------------------------------------------------- 
inline RowIterator::RowIterator(Board* b, const Square& s) 
   : 
      BoardIterator(b,s) 
{
}


//----------------------------------------------------------------- 
// RowIterator::operator++
//----------------------------------------------------------------- 
inline RowIterator& RowIterator::operator++() 
{
   ++theSquare.col();
   return *this;
}


//----------------------------------------------------------------- 
// ColIterator::ColIterator
//----------------------------------------------------------------- 
inline ColIterator::ColIterator(Board* b, const Square& s) 
   : 
      BoardIterator(b,s) 
{
}


//----------------------------------------------------------------- 
// ColIterator::operator++
//----------------------------------------------------------------- 
inline ColIterator& ColIterator::operator++() 
{
   ++theSquare.row();
   return *this;
}


//----------------------------------------------------------------- 
// BackDiagIterator::BackDiagIterator
//----------------------------------------------------------------- 
inline 
BackDiagIterator::BackDiagIterator(Board* b, const Square& s) 
   : 
      BoardIterator(b,s) 
{
}

   
//----------------------------------------------------------------- 
// BackDiagIterator::operator++
//----------------------------------------------------------------- 
inline BackDiagIterator& BackDiagIterator::operator++() 
{
   ++theSquare.row();
   ++theSquare.col();
         
   return *this;
}


//----------------------------------------------------------------- 
// FwdDiagIterator::FwdDiagIterator
//----------------------------------------------------------------- 
inline FwdDiagIterator::FwdDiagIterator(Board* b, const Square& s) 
   : 
      BoardIterator(b,s) 
{
}

   
//----------------------------------------------------------------- 
// FwdDiagIterator::operator++
//----------------------------------------------------------------- 
inline FwdDiagIterator& FwdDiagIterator::operator++() 
{
   ++theSquare.row();
   --theSquare.col();
         
   return *this;
}


} // namespace SoftTrac


#endif
