#include "tconfig.h"
#include <assert.h>
#include <sstream>
#include "BoardServer.h"
#include "util/Trace.h"
#include "util/tstring.h"

using namespace std;

namespace SoftTrac {


//---
// Register remote commands
//---
static BoardServer::ConstructCommand  cmd0("BoardServer::ConstructCommand");
static BoardServer::IsOccupiedCommand cmd1("BoardServer::IsOccupiedCommand");
static BoardServer::MoveCommand       cmd2("BoardServer::MoveCommand");


//---
// Register remote events
//---
static BoardServer::ChangeEvent       evt0("BoardServer::ChangeEvent");
static BoardServer::MoveCompleteEvent evt1("BoardServer::MoveCompleteEvent");
static BoardServer::CleanupEvent      evt2("BoardServer::CleanupEvent");


//---
// Register remote listeners
//---
static BoardServer::ChangeListener       lst0("BoardServer::ChangeListener");
static BoardServer::MoveCompleteListener lst1("BoardServer::MoveCompleteListener");
static BoardServer::CleanupListener      lst2("BoardServer::CleanupListener");


Board* BoardServer::theBoard = 0;

static Trace trace("BoardServer"/*, Trace::ON*/);


//-----------------------------------------------------------------------------
// BoardServer::subject
//
// This method constructs the subject with the given size.
//-----------------------------------------------------------------------------
Board* BoardServer::subject(int size)
{
   delete theBoard;
   theBoard = new Board(size);
   return theBoard;
}


//-----------------------------------------------------------------------------
// BoardServer::subject
//
// This method provides access to the subject.
//-----------------------------------------------------------------------------
Board* BoardServer::subject()
{
   return theBoard;
}


//-----------------------------------------------------------------------------
// BoardServer::ConstructCommand::ConstructCommand
//
// This constructor maps this c++ ConstructCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
BoardServer::ConstructCommand::ConstructCommand(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// BoardServer::ConstructCommand::realExecute
//
// This method executes the ConstructCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void BoardServer::ConstructCommand::realExecute()
{
   //---
   // Get incoming arguments
   //---
   int size = getInt("size");

   //---
   // Execute some method
   //---
   Board* myBoard = subject(size);

   //---
   // Put return arguments
   //---
   put(myBoard);
}


//-----------------------------------------------------------------------------
// BoardServer::IsOccupiedCommand::IsOccupiedCommand
//
// This constructor maps this c++ IsOccupiedCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
BoardServer::IsOccupiedCommand::IsOccupiedCommand(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// BoardServer::IsOccupiedCommand::realExecute
//
// This method executes the IsOccupiedCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void BoardServer::IsOccupiedCommand::realExecute()
{
   //---
   // Get incoming arguments
   //---
   int row = getInt("row");
   int col = getInt("col");

   Square square (row, col);

   //---
   // Execute some method
   //---
   Board* myBoard = subject();
   bool isOccupied = myBoard->isOccupied(square);

   //---
   // Put back any return arguments
   //---
   put("occupied", isOccupied);
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCommand::MoveCommand
//
// This constructor maps this c++ MoveCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
BoardServer::MoveCommand::MoveCommand(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCommand::realExecute
//
// This method executes the MoveCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void BoardServer::MoveCommand::realExecute()
{
   //---
   // Get incoming arguments
   //---
   vector<int> s = getIntArray("start");
   vector<int> f = getIntArray("finish");

   Square start (s[0], s[1]);
   Square finish(f[0], f[1]);

   //---
   // Execute some method
   //---
   Board* myBoard = subject();
   bool isValid = myBoard->move(start, finish);

   //---
   // Put back any return arguments
   //---
   put("valid", isValid);
}


//-----------------------------------------------------------------------------
// BoardServer::ChangeEvent::ChangeEvent
//
// This constructor maps this c++ ChangeEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
BoardServer::ChangeEvent::ChangeEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// BoardServer::ChangeEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void BoardServer::ChangeEvent::attach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   Board* myBoard = subject();

   //---
   // attach the listener
   //---
   myBoard->attachChangeEvent(lst);
}


//-----------------------------------------------------------------------------
// BoardServer::ChangeEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void BoardServer::ChangeEvent::detach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   Board* myBoard = subject();

   //---
   // detach the listener
   //---
   myBoard->detachChangeEvent(lst);
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCompleteEvent::MoveCompleteEvent
//
// This constructor maps this c++ MoveCompleteEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
BoardServer::MoveCompleteEvent::MoveCompleteEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCompleteEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void BoardServer::MoveCompleteEvent::attach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   Board* myBoard = subject();

   //---
   // attach the listener
   //---
   myBoard->attachMoveCompleteEvent(lst);
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCompleteEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void BoardServer::MoveCompleteEvent::detach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   Board* myBoard = subject();

   //---
   // detach the listener
   //---
   myBoard->detachMoveCompleteEvent(lst);
}


//-----------------------------------------------------------------------------
// BoardServer::CleanupEvent::CleanupEvent
//
// This constructor maps this c++ CleanupEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
BoardServer::CleanupEvent::CleanupEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// BoardServer::CleanupEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void BoardServer::CleanupEvent::attach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   Board* myBoard = subject();

   //---
   // attach the listener
   //---
   myBoard->attachCleanupEvent(lst);
}


//-----------------------------------------------------------------------------
// BoardServer::CleanupEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void BoardServer::CleanupEvent::detach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   Board* myBoard = subject();

   //---
   // detach the listener
   //---
   myBoard->detachCleanupEvent(lst);
}


//-----------------------------------------------------------------------------
// BoardServer::ChangeListener::ChangeListener
//
// This constructor maps this c++ ChangeListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
BoardServer::ChangeListener::ChangeListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// BoardServer::ChangeListener::update
//
// This constructor maps this c++ ChangeListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void BoardServer::ChangeListener::update(Event* event)
   throw (std::invalid_argument)
{
   Board::Event* myEvent = dynamic_cast<Board::Event*>(event);
   if (!myEvent) 
   {
      std::ostringstream what;
      what << event->name() << " events not associated with this subject";
      throw std::invalid_argument(what.str());
   }

   Board* board = myEvent->subject();

   //---
   // Put back any event arguments
   //---
   put("row",   myEvent->square().row());
   put("col",   myEvent->square().col());
   put("token", board->getToken(myEvent->square())->getToken());

   //---
   // Forward update across the JNI interface
   //---
   forwardNotify();
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCompleteListener::MoveCompleteListener
//
// This constructor maps this c++ MoveCompleteListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
BoardServer::MoveCompleteListener::MoveCompleteListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// BoardServer::MoveCompleteListener::update
//
// This constructor maps this c++ MoveCompleteListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void BoardServer::MoveCompleteListener::update(Event* event)
   throw (std::invalid_argument)
{
   forwardNotify();
}


//-----------------------------------------------------------------------------
// BoardServer::CleanupListener::CleanupListener
//
// This constructor maps this c++ CleanupListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
BoardServer::CleanupListener::CleanupListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// BoardServer::CleanupListener::update
//
// This constructor maps this c++ CleanupListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void BoardServer::CleanupListener::update(Event* event)
   throw (std::invalid_argument)
{
   static const string METHOD = "CleanupListener::update";

   Board::Event* myEvent = dynamic_cast<Board::Event*>(event);
   if (!myEvent) 
   {
      std::ostringstream what;
      what << event->name() << " events not associated with this subject";
      throw std::invalid_argument(what.str());
   }

   Board* board = myEvent->subject();

   // Get the score
   put("score", board->getScore());
   trace.debug(METHOD, toString(board->getScore()));

   //---
   // Get the state of the board
   //---
   char* tokenArray = new char[board->numCols()+1];
   tokenArray[board->numCols()] = 0;
   for (int n = 0; n < board->numRows(); ++n)
   {
      Row r = board->row(n);
      int m = 0;
      for (Row::iterator ri = r.begin(); ri != r.end(); ++ri, ++m)
      {
         tokenArray[m] = (*ri)->getToken();
      }
      put("row"+toString(n), tokenArray);
      trace.debug(METHOD, tokenArray);
   }
   delete[] tokenArray;

   trace.debug(METHOD, "\n"+toString(*board));
   forwardNotify();
}


} // namespace SoftTrac
