//----------------------------------------------------------------- 
// FILE: ComputerPlayer.cpp
//----------------------------------------------------------------- 

#include "tconfig.h"
#include <assert.h>
#include <algorithm>
#include <iostream>
#include <stdexcept>
#include <vector>
#include "ComputerPlayer.h"
#include "Board.h"
#include "BoardIterator.h"
#include "TokenGenerator.h"
#include "Token.h"

using namespace std;

namespace SoftTrac {

      
//----------------------------------------------------------------- 
// ComputerPlayer::ComputerPlayer
//----------------------------------------------------------------- 
ComputerPlayer::ComputerPlayer(Board* board)
   :
      Player        (board),
      theObserver   (this),
      theEvents     (this, "ComputerPlayer", 1), 
      theSkillLevel (1)
{
   int startTokens = 2*TOKENS_TO_PLAY+1;

   if (board) 
   {
      board->attachLevelChangeEvent(&theObserver);
      placeTokens(startTokens);
   }

   int tokensToChooseFrom = TOKENS_TO_PLAY + theSkillLevel;
   for (int i = 0; i < startTokens; ++i)
   {
      const Token* token = TokenGenerator::next(tokensToChooseFrom);
      theLookAheadQueue.push_back(token);
   }
}


//----------------------------------------------------------------- 
// ComputerPlayer::setBoard
//----------------------------------------------------------------- 
void ComputerPlayer::setBoard(Board* board)
{
   Player::setBoard(board);

   if (board) 
   {
      board->attachLevelChangeEvent(&theObserver);
      placeTokens(2*TOKENS_TO_PLAY+1);
   }
}


//----------------------------------------------------------------- 
// ComputerPlayer::nextMove
//----------------------------------------------------------------- 
void ComputerPlayer::nextMove()
{
   if (!theBoard->cleanup()) 
   {
      placeTokens(TOKENS_TO_PLAY);
      theBoard->cleanup();
   }
}


//----------------------------------------------------------------- 
// ComputerPlayer::LevelChangeObserver::update
//----------------------------------------------------------------- 
void ComputerPlayer::LevelChangeObserver::update(Event*) 
   throw (std::invalid_argument)
{
   theComputerPlayer->increaseDifficulty();
}


//----------------------------------------------------------------- 
// ComputerPlayer::LevelChangeObserver::name
//----------------------------------------------------------------- 
std::string ComputerPlayer::LevelChangeObserver::name() const 
{ 
   return "ComputerPlayer::LevelChangeObserver"; 
}


//----------------------------------------------------------------- 
// ComputerPlayer::placeTokens
//----------------------------------------------------------------- 
void ComputerPlayer::placeTokens(unsigned int tokensToPlay)
{
   std::vector<Square> blanks;
   
   //---
   // Find all the blank squares
   //---
   for (int i = 0; i < theBoard->numRows(); ++i)
   {
      int j = 0;
      Row r = theBoard->row(i);
      for (Row::iterator ri = r.begin(); ri != r.end(); ++ri)
      {
         if ((*ri)->isBlank()) blanks.push_back(Square(i,j));
         ++j;
      }
   }

   //---
   // Shuffle the choices
   //---
   std::random_shuffle(blanks.begin(), blanks.end());

   //---
   // Grab some tokens off the front
   //---
   int numTokens = (blanks.size() >= tokensToPlay) 
                 ? tokensToPlay
                 : blanks.size();

   if (!tokensToPlay) throw std::domain_error("Board is full");

   int tokensToChooseFrom = TOKENS_TO_PLAY + theSkillLevel;
   assert (numTokens <= theLookAheadQueue.size());
   for (int k = 0; k < numTokens; ++k)
   {
      const Token* token = theLookAheadQueue.back();
      theLookAheadQueue.pop_back();
      theEvents.notify(NEW_LOOKAHEAD);

      theBoard->putToken(blanks[k], token->getToken());
   }

   //---
   // Load up the lookahead queue for next time
   //---
   for (int j = 0; j < 3; ++j)
   {
      const Token* token = TokenGenerator::next(tokensToChooseFrom);
      theLookAheadQueue.push_back(token);
   }
   theEvents.notify(NEW_LOOKAHEAD);
}


} // namespace SoftTrac
