#include "tconfig.h"
#include <sstream>
#include "ComputerPlayerServer.h"
#include "nomess/Token.h"
#include "util/Trace.h"
#include "util/tstring.h"

#include <iostream>

using namespace std;

namespace SoftTrac {

static Trace trace("ComputerPlayerServer"/*, Trace::ON */);

//---
// Register remote commands
//---
static ComputerPlayerServer::ConstructCommand cmd0("ComputerPlayerServer::ConstructCommand");
static ComputerPlayerServer::NextMoveCommand  cmd1("ComputerPlayerServer::NextMoveCommand");

//---
// Register remote events/listeners
//---
static ComputerPlayerServer::NewLookAheadEvent    evt0("ComputerPlayerServer::NewLookAheadEvent");
static ComputerPlayerServer::NewLookAheadListener lst0("ComputerPlayerServer::NewLookAheadListener");


ComputerPlayer* ComputerPlayerServer::theComputerPlayer = 0;


//-----------------------------------------------------------------------------
// ComputerPlayerServer::subject
//
// This method provides access to the subject. The subject is
// created the first time the method is accessed.
//-----------------------------------------------------------------------------
ComputerPlayer* ComputerPlayerServer::subject()
{
   if (!theComputerPlayer) theComputerPlayer = new ComputerPlayer();
   return theComputerPlayer;
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::ConstructCommand::ConstructCommand
//
// This constructor maps this c++ ConstructCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
ComputerPlayerServer::ConstructCommand::ConstructCommand(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::ConstructCommand::realExecute
//
// This method executes the ConstructCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void ComputerPlayerServer::ConstructCommand::realExecute()
{
   const string METHOD = "ConstructCommand::realExecute";

   //---
   // Get incoming arguments
   //---
   void* ptr = getReference();
   Board* board = reinterpret_cast<Board*>(ptr);
   trace.debug(METHOD, "board "+toString(board));

   //---
   // Execute some method
   //---
   ComputerPlayer* myComputerPlayer = subject();
   myComputerPlayer->setBoard(board);
   trace.debug(METHOD, "myComputerPlayer "+toString(myComputerPlayer));

   //---
   // Put back any return arguments
   //---
   put(myComputerPlayer);
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NextMoveCommand::NextMoveCommand
//
// This constructor maps this c++ NextMoveCommand object with the
// given java command name.
//-----------------------------------------------------------------------------
ComputerPlayerServer::NextMoveCommand::NextMoveCommand(const std::string& cmd)
   :
      JavaCommand(cmd)
{
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NextMoveCommand::realExecute
//
// This method executes the NextMoveCommand on behalf of its java
// counterpart.
//-----------------------------------------------------------------------------
void ComputerPlayerServer::NextMoveCommand::realExecute()
{
   //---
   // Get incoming arguments
   //---
   // int value1 = getInt ("keyword1");
   // double value2 = getDouble("keyword2");

   //---
   // Execute some method
   //---
   ComputerPlayer* myComputerPlayer = subject();
   myComputerPlayer->nextMove();

   //---
   // Put back any return arguments
   //---
   // put("keyword3", myComputerPlayer->value3());
   // put("keyword4", myComputerPlayer->value4());
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NewLookAheadEvent::NewLookAheadEvent
//
// This constructor maps this c++ NewLookAheadEvent object with the
// given java event name.
//-----------------------------------------------------------------------------
ComputerPlayerServer::NewLookAheadEvent::NewLookAheadEvent(const std::string& evt)
   :
      JavaEvent(evt)
{
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NewLookAheadEvent::attach
//
// This method attaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void ComputerPlayerServer::NewLookAheadEvent::attach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   ComputerPlayer* myComputerPlayer = subject();

   //---
   // attach the listener
   //---
   myComputerPlayer->attachNewLookAheadEvent(lst);
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NewLookAheadEvent::detach
//
// This method detaches the given JNI listener and the subject.
//-----------------------------------------------------------------------------
void ComputerPlayerServer::NewLookAheadEvent::detach(JavaListener* lst)
{
   //---
   // Get the subject
   //---
   ComputerPlayer* myComputerPlayer = subject();

   //---
   // detach the listener
   //---
   myComputerPlayer->detachNewLookAheadEvent(lst);
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NewLookAheadListener::NewLookAheadListener
//
// This constructor maps this c++ NewLookAheadListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
ComputerPlayerServer::NewLookAheadListener::NewLookAheadListener(const std::string& lst)
   :
      JavaListener(lst)
{
}


//-----------------------------------------------------------------------------
// ComputerPlayerServer::NewLookAheadListener::update
//
// This constructor maps this c++ NewLookAheadListener object with the
// given java listener name.
//-----------------------------------------------------------------------------
void ComputerPlayerServer::NewLookAheadListener::update(Event* event)
   throw (std::invalid_argument)
{
   ComputerPlayer* myComputerPlayer = subject();
   vector<const Token*> lookAheadQueue = myComputerPlayer->lookAheadQueue();

   string lookAhead;
   for (int i = lookAheadQueue.size()-1; i >= 0; --i)
   {
      char c = lookAheadQueue[i]->getToken();
      lookAhead += c;
   }

   //---
   // Put back any event arguments
   //---
   put("lookahead", lookAhead);

   //---
   // Forward update across the JNI interface
   //---
   forwardNotify();
}


} // namespace SoftTrac
