//-----------------------------------------------------------------------------
// FILE:              TestHarness.cpp
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include "./TestHarness.h"
#include "./TestDriver.h"
#include "util/strings/Filename.h"

namespace SoftTrac {

TestHarness::Registry* TestHarness::registry = 0;


//-----------------------------------------------------------------------------
// TestHarness::TestHarness
//-----------------------------------------------------------------------------
TestHarness::TestHarness(int           argc, 
                         char**        argv,
                         std::ostream& infoStream,
                         std::ostream& resultsStream,
                         std::ostream& errorStream,
                         uint          abortAfter)
   :
      theProgramName      (Filename(argv[0]).baseName()),
      theArgc             (argc),
      theArgv             (argv),
      theInfoStream       (infoStream),
      theResultsStream    (resultsStream),
      theErrorStream      (errorStream),
      theMaxSecsPerDriver (abortAfter)
{
   // If no drivers are registered then return
   if (!registry) return;
   
   std::list<TestDriver*> drivers;

   //---
   // If no test drivers were specified on the command line,
   // then run them all
   //---
   if (argc == 1)
   {
      Registry::const_iterator i;
      for (i = registry->begin(); i != registry->end(); ++i)
      {
         TestDriver* driver = (*i).second;
         drivers.push_back(driver);
      }
   }

   //---
   // Otherwise, just run the requested drivers
   //---
   else
   {
      for (int i = 1; i < argc; ++i)
      {
         std::string driverName(argv[i]);
         TestDriver* driver = (*registry)[driverName];
         if (driver) drivers.push_back(driver);
      }
   }

   testDrivers(drivers);
}


//-----------------------------------------------------------------------------
// TestHarness::registerDriver
//-----------------------------------------------------------------------------
void TestHarness::registerDriver(TestDriver* driver)
{
   if (!registry) registry = new Registry();
   registry->insert(std::make_pair(driver->name(), driver));
}


//-----------------------------------------------------------------------------
// TestHarness::header
//-----------------------------------------------------------------------------
std::ostream& TestHarness::header(TestDriver* driver)
{
   theInfoStream
      << "\n-----------------------------------------------------------------\n"
      << "NAME:         \t" << driver->name() << "\n"
      << "START TIME:   \t??:??:??\n\n";
   
   return theInfoStream;
}


//-----------------------------------------------------------------------------
// TestHarness::footer
//-----------------------------------------------------------------------------
std::ostream& TestHarness::footer()
{
   theInfoStream
      << "\nSTOP TIME:   \t??:??:??\n"
      << "ELAPSED TIME:\t??:??:??\n"
      << "-----------------------------------------------------------------\n"
      << std::endl;

   return theInfoStream;
}


//-----------------------------------------------------------------------------
// WatchDog::WatchDog
//-----------------------------------------------------------------------------
TestHarness::WatchDog::WatchDog(uint seconds) 
{ 
//   alarm(seconds); 
}


//-----------------------------------------------------------------------------
// WatchDog::~WatchDog
//-----------------------------------------------------------------------------
TestHarness::WatchDog::~WatchDog() 
{
//   alarm(0);
}


//-----------------------------------------------------------------------------
// TestHarness::testDrivers
//-----------------------------------------------------------------------------
void TestHarness::testDrivers(const std::list<TestDriver*>& drivers)
{
   uint testsRun    = 0;
   uint testsPassed = 0;

   std::list<TestDriver*>::const_iterator driver;
   for (driver = drivers.begin(); driver != drivers.end(); ++driver)
   {
      header(*driver);

      try 
      {
         ++testsRun;
         WatchDog blackJack(theMaxSecsPerDriver);
         (*driver)->execute(this);
         ++testsPassed;
      }
      catch (const SoftTrac::Exception& e)
      {
         theErrorStream << e << std::endl;
         e.printStackTrace(theErrorStream) << std::endl;
         Exception::format(theErrorStream, __LINE__, __FILE__);
         theErrorStream << (*driver)->name() << " FAILED." << std::endl;
      } 
      catch (const std::exception& e)
      {
         theErrorStream << e.what() << std::endl;
         Exception::format(theErrorStream, __LINE__, __FILE__);
         theErrorStream << (*driver)->name() << " FAILED." << std::endl;
      }
      catch (...)
      {
         theErrorStream << "Unknown exception" << std::endl;
         Exception::format(theErrorStream, __LINE__, __FILE__);
         theErrorStream << (*driver)->name() << " FAILED." << std::endl;
      }

      footer();
   }

   theInfoStream << testsPassed << "/" << testsRun 
                 << " test drivers PASSED" << std::endl;
}


} // namespace SoftTrac
