//-----------------------------------------------------------------------------
// FILE:              Exception.cpp
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%

#include "tconfig.h"
#include <iostream>
#include <sstream>
#include "./Exception.h"
#include "util/strings/Filename.h"

using namespace std;

namespace SoftTrac {


//---------------------------------------------------------------------------
// Exception::~Exception
//---------------------------------------------------------------------------
Exception::~Exception() throw()
{
}


//---------------------------------------------------------------------------
// Exception::what
//---------------------------------------------------------------------------
const char* Exception::what() const
{
   return theWhat.c_str();
}


//---------------------------------------------------------------------------
// Exception::push
//---------------------------------------------------------------------------
void Exception::push(const char*        who,
                     int                line,
                     const std::string& file) const
{
   std::ostringstream os;
   os << who;
   format(os, line, file);

   theStack.push_back(os.str());
}
   

//---------------------------------------------------------------------------
// Exception::printStackTrace
//---------------------------------------------------------------------------
std::ostream& Exception::printStackTrace(std::ostream& os) const
{
   if (theStack.empty()) return os;

   os << "Stack Trace:";

   for (vector<string>::const_iterator i  = theStack.begin();
                                       i != theStack.end();
                                      ++i)
   {
      os << '\n' << *i;
   }

   return os;
}


//---------------------------------------------------------------------------
// Exception::format
//---------------------------------------------------------------------------
std::ostream& Exception::format(std::ostream&      os, 
                                int                line, 
                                const std::string& file) 
{
   os << " (";

   if (!file.empty())
   {
      os << "\"" << Filename(file).baseName() << "\"";
   }

   if (line)
   {
      os << ", line " << line;
   }
   
   os << ") ";

   return os;
}


//---------------------------------------------------------------------------
// Exception::Exception
//---------------------------------------------------------------------------
Exception::Exception(const string& name,
                     const string& reason,
                     const string& who,
                     int           line,
                     const string& file) throw()
   :
      theName   (name),
      theReason (reason),
      theWho    (who),
      theLine   (line),
      theFile   (file)
{try
{
   setWhat(reason);
}
catch(...)
{
}}


//---------------------------------------------------------------------------
// Exception::setWhat
//---------------------------------------------------------------------------
void Exception::setWhat(const std::string& reason) throw()
{try
{
   theReason = reason;

   std::ostringstream s;
   s << theName << " -- " << reason << " in " << theWho;
   format(s, theLine, theFile);

   theWhat = s.str();
}
catch(...)
{
}}


//---------------------------------------------------------------------------
// LogicError::LogicError
//---------------------------------------------------------------------------
LogicError::LogicError(const string& name,
	                    const string& what,
	                    const string& who,
	                    int           line,
	                    const string& file) throw()
   :
      Exception(name, what, who, line, file)
{
}


//---------------------------------------------------------------------------
// DomainError::DomainError
//---------------------------------------------------------------------------
DomainError::DomainError(const string& what,
	                      const string& who,
	                      int           line,
	                      const string& file) throw()
   :
      LogicError   ("DomainError", what, who, line, file),
      domain_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// DomainError::DomainError
//---------------------------------------------------------------------------
DomainError::DomainError(const string& name,
	                      const string& what,
	                      const string& who,
	                      int           line,
	                      const string& file) throw()
   :
      LogicError   (name, what, who, line, file),
      domain_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// InvalidArgument::InvalidArgument
//---------------------------------------------------------------------------
InvalidArgument::InvalidArgument(const string& what,
	                              const string& who,
	                              int           line,
	                              const string& file) throw()
   :
      LogicError       ("InvalidArgument", what, who, line, file),
      invalid_argument (what.c_str())
{
}


//---------------------------------------------------------------------------
// InvalidArgument::InvalidArgument
//---------------------------------------------------------------------------
InvalidArgument::InvalidArgument(const string& name,
	                              const string& what,
	                              const string& who,
	                              int           line,
	                              const string& file) throw()
   :
      LogicError       (name, what, who, line, file),
      invalid_argument (what.c_str())
{
}


//---------------------------------------------------------------------------
// LengthError::LengthError
//---------------------------------------------------------------------------
LengthError::LengthError(const string& what,
	                      const string& who,
	                      int           line,
	                      const string& file) throw()
   :
      LogicError   ("LengthError", what, who, line, file),
      length_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// LengthError::LengthError
//---------------------------------------------------------------------------
LengthError::LengthError(const string& name,
	                      const string& what,
	                      const string& who,
	                      int           line,
	                      const string& file) throw()
   :
      LogicError   (name, what, who, line, file),
      length_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// OutOfRange::OutOfRange
//---------------------------------------------------------------------------
OutOfRange::OutOfRange(const string& what,
	                    const string& who,
	                    int           line,
	                    const string& file) throw()
   :
      LogicError   ("OutOfRange", what, who, line, file),
      out_of_range (what.c_str())
{
}


//---------------------------------------------------------------------------
// OutOfRange::OutOfRange
//---------------------------------------------------------------------------
OutOfRange::OutOfRange(const string& name,
	                    const string& what,
	                    const string& who,
	                    int           line,
	                    const string& file) throw()
   :
      LogicError   (name, what, who, line, file),
      out_of_range (what.c_str())
{
}


//---------------------------------------------------------------------------
// RuntimeError::RuntimeError
//---------------------------------------------------------------------------
RuntimeError::RuntimeError(const string& name,
	                        const string& what,
	                        const string& who,
	                        int           line,
	                        const string& file) throw()
   :
      Exception(name, what, who, line, file)
{
}


//---------------------------------------------------------------------------
// RangeError::RangeError
//---------------------------------------------------------------------------
RangeError::RangeError(const string& what,
	                    const string& who,
	                    int           line,
	                    const string& file) throw()
   :
      RuntimeError ("RangeError", what, who, line, file),
      range_error  (what.c_str())
{
}


//---------------------------------------------------------------------------
// RangeError::RangeError
//---------------------------------------------------------------------------
RangeError::RangeError(const string& name,
	                    const string& what,
	                    const string& who,
	                    int           line,
	                    const string& file) throw()
   :
      RuntimeError (name, what, who, line, file),
      range_error  (what.c_str())
{
}


//---------------------------------------------------------------------------
// OverflowError::OverflowError
//---------------------------------------------------------------------------
OverflowError::OverflowError(const string& what,
	                          const string& who,
	                          int           line,
	                          const string& file) throw()
   :
      RuntimeError   ("OverflowError", what, who, line, file),
      overflow_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// OverflowError::OverflowError
//---------------------------------------------------------------------------
OverflowError::OverflowError(const string& name,
	                          const string& what,
	                          const string& who,
	                          int           line,
	                          const string& file) throw()
   :
      RuntimeError   (name, what, who, line, file),
      overflow_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// UnderflowError::UnderflowError
//---------------------------------------------------------------------------
UnderflowError::UnderflowError(const string& what,
	                            const string& who,
	                            int           line,
	                            const string& file) throw()
   :
      RuntimeError    ("UnderflowError", what, who, line, file),
      underflow_error (what.c_str())
{
}


//---------------------------------------------------------------------------
// UnderflowError::UnderflowError
//---------------------------------------------------------------------------
UnderflowError::UnderflowError(const string& name,
	                            const string& what,
	                            const string& who,
	                            int           line,
	                            const string& file) throw()
   :
      RuntimeError    (name, what, who, line, file),
      underflow_error (what.c_str())
{
}


} // namespace SoftTrac


//---------------------------------------------------------------------------
// operator<<
//---------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const SoftTrac::Exception& e)
{
   return os << e.what();
}



