//-----------------------------------------------------------------------------
// FILE:              Exception.h
//
//    Copyright (C)  2000 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_Exception_H
#define SoftTrac_Exception_H

#include <iosfwd>
#include <stdexcept>
#include <string>
#include <vector>
#include "util/util_api.h"

namespace SoftTrac {

//---------------------------------------------------------------------------
// CLASS: Exception
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API Exception
{
public:
   virtual ~Exception() throw();

   virtual const char* what() const;

   const std::string& name()   const { return theName;   }
   const std::string& reason() const { return theReason; }

   void push(const char*        who,
             int                line=0,
             const std::string& file="") const;

   std::ostream& printStackTrace(std::ostream&) const; 

   static std::ostream& format(std::ostream&      os,
                               int                line,
                               const std::string& file);

protected:
   Exception(const std::string& name,
             const std::string& reason,
             const std::string& who,
             int                line,
             const std::string& file) throw();

   void setWhat(const std::string& reason) throw();

private:
   std::string theName;
   std::string theWhat;
   std::string theReason;
   std::string theWho;
   int         theLine;
   std::string theFile;

   mutable std::vector<std::string> theStack;
};


//---------------------------------------------------------------------------
// CLASS: LogicError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API LogicError : public Exception
{
protected:
   LogicError(const std::string& name,
              const std::string& reason,
              const std::string& who,
              int                line=0,
              const std::string& file="") throw();
};


//---------------------------------------------------------------------------
// CLASS: DomainError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API DomainError : public LogicError,
                                      public std::domain_error
{
public:
   DomainError(const std::string& reason, 
		         const std::string& who,  
               int                line=0,
               const std::string& file="") throw();

protected:
   DomainError(const std::string& name,
	            const std::string& reason,
					const std::string& who,
               int                line,
               const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS: InvalidArgument
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API InvalidArgument : public LogicError,
                                          public std::invalid_argument
{
public:
   InvalidArgument(const std::string& reason,
	                const std::string& who,
                   int                line=0,
                   const std::string& file="") throw();

protected:
   InvalidArgument(const std::string& name,
	                const std::string& reason,
                   const std::string& who,
                   int                line,
                   const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS: LengthError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API LengthError : public LogicError,
                                      public std::length_error
{
public:
   LengthError(const std::string& reason,
	          	const std::string& who,
               int                line=0,
               const std::string& file="") throw();

protected:
   LengthError(const std::string& name,
	            const std::string& reason,
               const std::string& who,
               int                line,
               const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS: OutOfRange
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API OutOfRange : public LogicError,
                                     public std::out_of_range
{
public:
   OutOfRange(const std::string& reason,
	           const std::string& who,
              int                line=0,
              const std::string& file="") throw();

protected:
   OutOfRange(const std::string& name,
	           const std::string& reason,
              const std::string& who,
              int                line,
              const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS: RuntimeError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API RuntimeError : public Exception
{
protected:
   RuntimeError(const std::string& name,
	             const std::string& reason,
                const std::string& who,
                int                line=0,
                const std::string& file="") throw();
};


//---------------------------------------------------------------------------
// CLASS: RangeError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API RangeError : public RuntimeError,
                                     public std::range_error
{
public:
   RangeError(const std::string& reason,
		        const std::string& who,
              int                line=0,
              const std::string& file="") throw();

protected:
   RangeError(const std::string& name,
		        const std::string& reason,
		        const std::string& who,
              int                line,
              const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS: OverflowError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API OverflowError : public RuntimeError,
                                        public std::overflow_error
{
public:
   OverflowError(const std::string& reason,
	              const std::string& who,
                 int                line=0,
                 const std::string& file="") throw();

protected:
   OverflowError(const std::string& name,
	              const std::string& reason,
                 const std::string& who,
                 int                line,
                 const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS: UnderflowError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API UnderflowError : public RuntimeError,
                                         public std::underflow_error
{
public:
   UnderflowError(const std::string& reason,
	               const std::string& who,
                  int                line=0,
                  const std::string& file="") throw();

protected:
   UnderflowError(const std::string& name,
		            const std::string& reason,
		            const std::string& who,
                  int                line,
                  const std::string& file) throw();
};


//---------------------------------------------------------------------------
// CLASS:  RethrowError
//---------------------------------------------------------------------------
class SOFTTRAC_UTIL_API RethrowError : public DomainError
{
public:
   RethrowError(const std::exception& exception,
                const std::string&    who,
                int                   line=0,
                const std::string&    file="") throw()
      : DomainError("RethrowError", exception.what(), who, line, file) {}
   
   RethrowError(const SoftTrac::Exception& exception,
                const std::string&         reason,
                const std::string&         who,
                int                        line=0,
                const std::string&         file="")
      : DomainError(exception.name(),
                    exception.reason()+" "+reason, who, line, file) {}
   
   RethrowError(const std::string&         reason,
                const std::string&         who,
                int                        line=0,
                const std::string&         file="")
      : DomainError("RethrowError", reason, who, line, file) {}
   
   RethrowError(const std::string&         who,
                int                        line=0,
                const std::string&         file="")
      : DomainError("RethrowError", "Unknown error", who, line, file) {}
};


//---------------------------------------------------------------------------
// CLASS:  NamedException
//---------------------------------------------------------------------------
template <class BaseException>
class NamedException : public BaseException
{ 
public: 
   NamedException(const std::string& name, 
                  const std::string& reason, 
                  const std::string& who,
                  int                line=0,
                  const std::string& file="")
      : BaseException(name, reason, who, line, file) {}

   virtual ~NamedException() throw() {}
};


//---------------------------------------------------------------------------
// TRY_EXCEPTION
//---------------------------------------------------------------------------
#define TRY_EXCEPTION {try


//---------------------------------------------------------------------------
// RETHROW_EXCEPTION
//---------------------------------------------------------------------------
#define RETHROW_EXCEPTION(MODULE) \
catch(const SoftTrac::Exception& e) \
{ \
   e.push(MODULE, __LINE__, __FILE__); \
   throw; \
} \
catch(const std::exception& e) \
{ \
   throw SoftTrac::RethrowError(e, MODULE, __LINE__, __FILE__); \
} \
catch(...) \
{ \
   throw SoftTrac::RethrowError(MODULE, __LINE__, __FILE__); \
}}


//---------------------------------------------------------------------------
// CATCHALL_EXCEPTION
//---------------------------------------------------------------------------
#define CATCHALL_EXCEPTION \
catch(const SoftTrac::Exception& e) \
{ \
   std::cerr << e << std::endl; \
   e.printStackTrace(std::cerr) << std::endl; \
   return 1; \
} \
catch(const std::exception& e) \
{ \
   std::cerr << e.what() << std::endl; \
   return 1; \
} \
catch(...) \
{ \
   std::cerr << "Unknown exception" << std::endl; \
   return 1; \
}}


} // namespace SoftTrac


std::ostream& operator<<(std::ostream&, const SoftTrac::Exception&);


#endif
