//-----------------------------------------------------------------------------
// FILE:              VectorT.h
//
//    Copyright (C)  2001 SoftTrac Corporation. All rights reserved.
//-----------------------------------------------------------------------------
// %W% %G%
#ifndef SoftTrac_VectorT_H
#define SoftTrac_VectorT_H

#include "tconfig.h"
#include <iosfwd>
#include "util/exceptions/CommonExceptions.h"
#include "util/talgorithm.h"
#include "util/tfunctional.h"

namespace SoftTrac {

//---------------------------------------------------------------------------
// CLASS:  VectorT
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
class VectorT
{
public:
	typedef const T* const_iterator;
	typedef T*       iterator;

protected:
	VectorT(){}
	VectorT(const T& t0, const T& t1);
	VectorT(const T& t0, const T& t1, const T& t2);
	VectorT(const_iterator first, const_iterator last) { copy(first, last); }

public:
	const T& operator[](unsigned int n) const;
	T&       operator[](unsigned int n);

	const T& at(int n) const { return theCoord[n]; }
	T&       at(int n)       { return theCoord[n]; }

	VectorT& operator+=(const VT&);
	VectorT& operator-=(const VT&);
	VectorT& operator*=(const T&);
	VectorT& operator/=(const T&);

   VT  operator+ (const VT&) const;
	VT  operator- (const VT&) const;
	VT  operator* (const T&)  const;
	VT  operator/ (const T&)  const;

	void copy(const_iterator first, const_iterator last)
   { std::copy(first, last, theCoord); }

   const_iterator   begin() const { return theCoord; }
	iterator         begin()       { return theCoord; }
	const_iterator   end()   const { return theCoord + DIM; }
	iterator         end()         { return theCoord + DIM; }

private:
	T theCoord[DIM];
};


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::VectorT
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VectorT<VT,T,DIM>::VectorT(const T& t0, const T& t1)
{
	theCoord[0] = t0;
	theCoord[1] = t1;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::VectorT
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VectorT<VT,T,DIM>::VectorT(const T& t0, const T& t1, const T& t2)
{
	theCoord[0] = t0;
	theCoord[1] = t1;
	theCoord[2] = t2;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator[]
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline const T& VectorT<VT,T,DIM>::operator[](unsigned int n) const
{
	return const_cast<VectorT<VT,T,DIM>*>(this)->operator[](n);
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator[]
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline T& VectorT<VT,T,DIM>::operator[](unsigned int n)
{
   if (n >= DIM) throw range_error("Index of out range");
   return theCoord[n];
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator+=
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VectorT<VT,T,DIM>& VectorT<VT,T,DIM>::operator+=(const VT& vt)
{
	forEach(begin(), end(), vt.begin(), BinaryPlusEqual<T>());
	return *this;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator-=
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VectorT<VT,T,DIM>& VectorT<VT,T,DIM>::operator-=(const VT& vt)
{
	forEach(begin(), end(), vt.begin(), BinaryMinusEqual<T>());
	return *this;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator*=
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VectorT<VT,T,DIM>& VectorT<VT,T,DIM>::operator*=(const T& t)
{
	for_each(begin(), end(), UnaryMultiplyEqual<T>(t));
	return *this);
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator/=
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VectorT<VT,T,DIM>& VectorT<VT,T,DIM>::operator/=(const T& t)
{
	for_each(begin(), end(), UnaryDivideEqual<T>(t));
	return *this);
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator+
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VT VectorT<VT,T,DIM>::operator+(const VT& vt) const
{
	VT v(*reinterpret_cast<const VT*>(this));
	v += vt;

	return v;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator-
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VT VectorT<VT,T,DIM>::operator-(const VT& vt) const
{
	VT v(*reinterpret_cast<const VT*>(this));
	v -= vt;

	return v;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator*
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VT VectorT<VT,T,DIM>::operator*(const T& t) const
{
	VT v(*reinterpret_cast<const VT*>(this));
	v *= t;

	return v;
}


//---------------------------------------------------------------------------
// VectorT<VT,T,DIM>::operator/
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
inline VT VectorT<VT,T,DIM>::operator/(const T& t) const
{
	VT v(*reinterpret_cast<const VT*>(this));
	v /= t;

	return v;
}


} // namespace SoftTrac


//---------------------------------------------------------------------------
// operator<<
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
std::ostream& 
operator<<(std::ostream& os, const SoftTrac::VectorT<VT,T,DIM>& vt)
{
   os << '(';
   SoftTrac::print_elements(vt.begin(), vt.end(), os, ' ', ' ', ' ');
   os << ')';

   return os;
}


//---------------------------------------------------------------------------
// operator>>
//---------------------------------------------------------------------------
template <class VT, class T, int DIM>
std::istream& operator>>(std::istream& is, SoftTrac::VectorT<VT,T,DIM>& vt)
{
   static const char* const MODULE = "operator>>(istream&, VectorT<VT,T,DIM>&";

   using namespace SoftTrac;

   char delimiter;

   is >> delimiter;
   for (uint i = 0; i < DIM; ++i) is >> vt.at(i);
   is >> delimiter;

   if (!is) throw SoftTrac::FormatError("Bad istream", 
                                        MODULE, __LINE__, __FILE__);
   return is;
}


#endif

